;
/**
 * <pre>
 * Coralian 的前台模块
 * 内容都是于当前网站的具体业务逻辑无关，可以通用于所有平台。
 * 具体实现 最基本的包括封装 ajax，实现前台数据验证的逻辑、动画在内的浏览器操作。
 * </pre>
 */

Coralian.browserOnly();
Coralian.client = (function () {
	/*
	 * 因为客户端 会用到 HTMLCollection 这些只有浏览器中才会出现的元素，
	 * 所以先把 forEach 这个方法覆盖掉
	 */
	Coralian.util.ObjectUtil.override(Object, 'forEach', function (method) {
		return function (obj, callback) {
			if (obj instanceof HTMLCollection) {
				for (let i = 0, len = obj.length; i < len; i++) {
					let result = callback(i, obj[i]);
					if (false === result) break;
					if (true === result) continue;
				}
			} else {
				return method(obj, callback);
			}
		}
	});

	var doc = document;
	// 设置 JSON 的两个处理
	var JSONparse = JSON.parse,
		queryString = Coralian.util.StringUtil.queryString;
	var {unsupportedType, unsupportedOperation, errorCast, illegalArguments} = Error;
	var isNumber = Number.isNumber,
		isArray = Array.isArray,
		doNothing = Function.EMPTY_BODY;
	var {newXmlWrapper, isXmlWrapper} = Coralian.dom;

	function newWindow() {
		return window.open.apply(null, arguments);
	}

	function getScrollTop() {
		return document.body.scrollTop || document.documentElement.scrollTop;
	}

	var $dom = (function () {

		var INNER_HTML = 'innerHTML',
			OUTER_HTML = 'outerHTML',
			VALUE = 'value',
			STYLE = 'style';

		function getNode(idName) {
			return doc.getElementById(idName);
		}

		function dom() {

			var elements = {}, names = [],
				size = 0;

			function getNodeName(index) {

				index = index || 0;

				if (isNumber(index)) {
					return names[index];
				}
				return index;
			}

			function createNode(obj) {

				var element, id = null;
				if (typeIs(obj, 'string')) {
					element = getNode(obj);
					id = obj;
				} else if (obj instanceof HTMLElement) {
					element = obj;
					id = obj.id;
				} else {
					unsupportedType(obj);
				}

				names[size] = id;
				elements[id] = element;
				size++;
			}

			function add(objs) {
				if (typeIs(objs, 'string') || (objs instanceof HTMLElement)) {
					createNode(objs);
				} else if (objs.length === 1) {
					objs = objs[0];
					if (typeIs(objs, 'string') || (objs instanceof HTMLElement)) {
						createNode(objs);
					} else {
						if (!objs) {
							alert(new Error().stack);
						}
						for (var i = 0, len = objs.length; i < len; i++) {
							createNode(objs[i]);
						}
					}
				} else {
					for (var i = 0, len = objs.length; i < len; i++) {
						createNode(objs[i]);
					}
				}
			}
			add(arguments);

			var instance = {
				/**
				 * 该函数支持两个参数，分别为 index（下标号）和 property（想要取得的对象属性名）
				 * 可以只传入  index 或者 property，也可以都传入，或者什么都不传入。
				 * 不穿入 index 时，默认取第 0 件
				 * 不传入 property 时，默认取 innerHTML
				 * 
				 * 重载模式
				 * Object get() {
				 *     return elements[0].innerHTML;
				 * }
				 * Object get(String property) {
				 *     return elements[0][property];
				 * }
				 * Object get(int index) {
				 *     return elements[index].innerHTML;
				 * }
				 * Object get(int index, String property) {
				 *    return return elements[index][property];
				 * }
				 * Object get(String name, String type) {
				 *    return return elements[name][property];
				 * }
				 */
				get: function (index, property) {
					switch (arguments.length) {
						case 0:
							index = 0;
							property = INNER_HTML;
							break;
						case 1:
							if (Number.isNumber(index)) {
								property = INNER_HTML;
							} else {
								property = index;
								index = 0;
							}
							break;
						case 2:
							index = index || 0;
							property = property || INNER_HTML;
							break;
						default:
							illegalArguments();
					}

					if (!typeIs(property, 'string')) errorCast(property, String);

					return elements[getNodeName(index)][property];
				},
				/**
				 * 该函数支持三个参数，第一个为需要被设定的 HTML 字符串
				 * 后两个分别为 index（下标号）和 property（想要取得的对象属性名）
				 * 可以只传入 index 或者 type，也可以都传入。
				 * 不穿入 index 时，默认设置到第 0 件 不传入 property 时，默认设置到 innerHTML
				 * 
				 * 重载模式
				 * $ set(String html) {
				 *     elements[0].innerHTML = html;
				 * }
				 * $ set(int index, String html) {
				 *     elements[index].innerHTML = html;
				 * }
				 * $ set(String name, String html) {
				 *     elements[name].innerHTML = html;
				 * }
				 * $ set(int index, String property, String value) {
				 *     elements[index][property] = html;
				 * }
				 * $ set(String name, String property, String value) {
				 *     elements[name][property] = html;
				 * }
				 * $ set(int index, Map value) {
				 *     Object.addAll(value, elements[index]);
				 * }
				 * $ set(String name, Map value) {
				 *     Object.addAll(value, elements[name]);
				 * }
				 */
				set: function (index, property, value) {
					var isString = true;
					switch (arguments.length) {
						case 1: // 只有一个参数：value（index = 0；property = innerHTML）
							value = index;
							index = 0;
							property = INNER_HTML;
							break;
						case 2:
							// 有两个参数：name, value
							value = property;
							switch (typeOf(value)) {
								case 'string':
									// typeIs(value, 'string'),（property = innerHTML）
									property = INNER_HTML;
									break;
								case 'object':
									// typeIs(value, 'Object')
									isString = false;
									break;
								default:
									illegalArguments();
							}
							break;
						case 3:
							index = index || 0;
							property = property || INNER_HTML;
							break;
						default:
							illegalArguments();
					}

					try {
						if (isString) {
							if (!typeIs(property, 'string')) errorCast(property, String);
							elements[getNodeName(index)][property] = value;
						} else {
							Object.addAll(value, elements[getNodeName(index)]);
						}
					} catch (e) {
						alert(e.stack);
						alert(getNodeName(index) + "不存在");
					}
					return this;
				},
				/*
				 * 用于简化 set(get) 操作
				 */
				add: function (index, property, value) {

					var node = elements[getNodeName(index)];
					node[property] = value;

				},
				/*
				 * 简化 set(get()) 操作 
				 * TODO 因为使用的是 + ，所以目前只支持 String 对象
				 */
				append: function (value, index, property) {
					this.set(index, property, this.get(index, property) + value);
					return this;
				},
				isEmpty: function () {
					return size === 0;
				},
				size: function () {
					return size;
				},
				/**
				 * 给元素添加如  onclick 这样的操作函数
				 * 
				 * 重载模式
				 * on(String action, Function callback) {
				 *     elements[0]['on' + action] = callback;
				 * }
				 * on(int index, String action, Function callback) {
				 *     elements[index]['on' + action] = callback;
				 * }
				 * on(String index, String action, Function callback) {
				 *     elements[index]['on' + action] = callback;
				 * }
				 */
				on: function (index, action, callback) {
					switch (arguments.length) {
						case 2:
							action = index;
							callback = action;
							index = 0;
							break;
						case 3:
							break;
						default:
							illegalArguments();
					}
					if (!String.startsWith(action, 'on')) {
						action = 'on' + action;
					}
					var e = elements[getNodeName(index)];
					if (!e) {
						throw new Error();
					}
					e[action] = callback;
					return this;
				},
				/**
				 * 直接调用既存的方法
				 * 比如：
				 * event('onclick');
				 * 
				 * 重载模式
				 * event(String method) {
				 *     elements[0][method]();
				 * }
				 * event(int index, String method) {
				 *     elements[index][method]();
				 * }
				 */
				event: function (index, method) {
					switch (arguments.length) {
						case 1:
							method = index;
							index = 0;
							break;
						case 2:
							break;
						default:
							illegalArguments();
					}

					elements[getNodeName(index)][method]();
				},
				/**
				 * 类似于这样的重载：
				 * appendChild(int index,HTMLELement child) {
				 *     elements[index].appendChild(child);
				 * }
				 * appendChild(String name,HTMLELement child) {
				 *     elements[index].appendChild(child);
				 * }
				 * appendChild(HTMLELement child) {
				 *     elements[0].appendChild(child);
				 * }
				 * appendChild(int index,XmlWrapper child) {
				 *     elements[index].appendChild(child);
				 * }
				 * appendChild(String name,XmlWrapper child) {
				 *     elements[index].appendChild(child);
				 * }
				 * appendChild(XmlWrapper child) {
				 *     elements[0].appendChild(child);
				 * }
				 */
				appendChild: function (index, child) {
					if (arguments.length === 1) {
						child = index;
						index = 0;
					}
					if (isXmlWrapper(child)) {
						child = child.render();
					} else {
						unsupportedType(child);
					}

					try {
						elements[getNodeName(index)].appendChild(child);
					} catch (e) {
						alert(getNodeName(index) + "不存在");
					}

					return this;
				},
				getChild: function (index, childPkg, property) {

					return elements[getNodeName(index)];
				},
				removeChild: function (index) {

				},
				newChild: function (index) {
					var element = elements[getNodeName(index)];
					var child = element.createChild();
					return child;
				},
				clear: function () {
					elements = {};
					names = [];
				},
				add: add,
				remove: function (index) {

					delete elements[getNodeName(index)];
					delete names[index];
					size--;
				},
				/**
				 * 类似于这样的重载：
				 * css(int index, Map style) {
				 *     elements[index].style.addAll(style);
				 * }
				 * css(String index, Map style) {
				 *     elements[index].style.addAll(style);
				 * }
				 * css(Map style) {
				 *     elements[0].style.addAll(style);
				 * }
				 */
				css: function (index, style) {
					if (arguments.length === 1) {
						style = index;
						index = 0;
					}

					if (!typeOf(style)) {
						unsupportedType(style);
					}
					if (!typeIs(index, "number", "string")) {
						unsupportedType(index);
					}
					var element = elements[getNodeName(index)];

					if (!element) {
						throw new Error('没有元素');
					}

					Object.addAll(style, element.style);

					return this;
				},
				getID: function (index) {
					index = index || 0;
					if (isNaN(index)) {
						errorCast(index, Number);
					}
					return names[index];
				},
				getAllID: function () {
					return Array.clone(names);
				},
				reset: function (index) {
					var element = elements[getNodeName(index)].reset();
				}
			}

			Object.instanceTo(instance, dom);

			return instance;
		}

		Object.defineProperty(dom, 'INNER_HTML', {
			value: INNER_HTML,
			writable: false
		});
		Object.defineProperty(dom, 'OUTER_HTML', {
			value: OUTER_HTML,
			writable: false
		});
		Object.defineProperty(dom, 'VALUE', {
			value: VALUE,
			writable: false
		});
		Object.defineProperty(dom, 'STYLE', {
			value: STYLE,
			writable: false
		});
		Object.defineProperty(dom, 'on', {
			value: function (id, name, funcName) {
				getNode(id)[on + name] = funcName;
			},
			writable: false
		});
		Object.defineProperty(dom, 'set', {
			value: function (id, value, type) {
				try {
					getNode(id)[type || INNER_HTML] = value;
				} catch (e) {
					alert(e.stack);
				}
			},
			writable: false
		});
		Object.defineProperty(dom, 'get', {
			value: function (id, type) {
				return getNode(id)[type || INNER_HTML];
			},
			writable: false
		});
		Object.defineProperty(dom, 'appendChild', {
			value: function (id, child) {
				return getNode(id).appendChild(child);
			},
			writable: false
		});
		Object.defineProperty(dom, 'getNode', {
			value: getNode,
			writable: false
		});
		Object.defineProperty(dom, 'exists', {
			value: function (id) {
				let node = getNode(id);
				return node !== null;
			},
			writeable: false
		})
		Object.defineProperty(dom, 'hideElement', {
			value: function (id) {
				getNode(id).style.display = 'none';
			},
			writable: false
		});
		Object.defineProperty(dom, 'showElement', {
			value: function (id) {
				getNode(id).style.display = 'block';
			},
			writable: false
		});
		Object.defineProperty(dom, 'removeElement', {
			value: function (id) {
				var e = getNode(id);
				e.parentElement.removeChild(e);
			},
			writable: false
		});
		Object.defineProperty(dom, 'getTitle', {
			value: function () {
				return doc.title;
			},
			writable: false
		});
		Object.defineProperty(dom, 'setTitle', {
			value: function (title) {
				doc.title = title;
			},
			writable: false
		});
		Object.defineProperty(dom, 'create', {
			value: function (tag, property) {
				if (!tag) illegalArguments();
				if (!typeIs(tag, "string")) unsupportedType(tag);

				var node = doc.createElement(tag);
				if (property) {
					Object.addAll(property, node);
				}
				return node;
			},
			writable: false
		});
		Object.defineProperty(dom, 'getNodeByTagName', {
			value: function (tag, index) {
				index = index || 0;
				return doc.getElementsByTagName(tag)[index];
			},
			writeable: false
		});
		Object.defineProperty(dom, 'getNodesByTagName', {
			value: function (tag) {
				return doc.getElementsByTagName(tag);
			},
			writeable: false
		});
		Object.defineProperty(dom, 'getNodeByClassName', {
			value: function (className, index) {
				index = index || 0;
				return doc.getElementsByClassName(className)[index];
			},
			writeable: false
		});
		Object.defineProperty(dom, 'getNodesByClassName', {
			value: function (className) {
				return doc.getElementsByClassName(className)
			},
			writeable: false
		});


		return dom;
	})();

	var __ajax = (function () {

		let { HttpRequestMethod, HttpStatusCode, MimeType } = Coralian.constants;

		function ajax(arg) {

			var url = arg.url,
				method = arg.method || HttpRequestMethod.GET,
				data = arg.data;
			var success = arg.success || doNothing,
				fail = arg.fail || doNothing,
				parse = arg.parse || false,
				ignoreWarning = arg.ignoreWarning || false;
			var xhr = new XMLHttpRequest();

			if (null !== data && undefined !== data) {
				data = queryString(data);
			}

			if (method === HttpRequestMethod.GET && !!data) {
				url += "?" + data;
			}

			if (ignoreWarning && (!String.startsWith(url, "/") && !String.startsWith(url, ".")) &&
				!confirm('所请求的地址可能含有危害您电脑安全的信息，否继续载入？')) return;

			// 超时处理，当请求在 120 秒之后还没反应的时候，判断 HTTP 请求超时
			xhr.timeout = 120000;
			xhr.ontimeout = () => {
				xhr.abort();
				xhr.onreadystatechange = doNothing;
				var timeout = arg.timeout || doNothing;
				timeout();
			}

			xhr.onreadystatechange = function () {

				if (xhr.readyState !== 4) return;

				let contentType = xhr.getResponseHeader("Content-Type");
				let mime = contentType.split(";")[0];

				let req;
				if (parse || mime === MimeType.JSON) {
					try {
						req = JSONparse(xhr.responseText);
					} catch (e) {
						alert("不是有效的JSON格式。");
						return;
					}
				} else {
					if (responseText.contains('<script') && !confirm('页面包含不安全元素，是否继续载入？')) return;
					req = xhr.responseText;
				}

				switch (xhr.status) {
					case HttpStatusCode.OK:
						success(req);
						break;
					case HttpStatusCode.FORBIDDEN:
						alert(req.message);
						break;
					case HttpStatusCode.NOT_FOUND:
						alert("找不到请求的地址");
						break;
					case HttpStatusCode.GATEWAY_TIMEOUT:
						ajaxTimeout();
						break;
					default:
						fail();
				}
			};
			xhr.open(method, url, true);

			switch (method) {
				case HttpRequestMethod.GET:
					break;
				case HttpRequestMethod.PUT:
				case HttpRequestMethod.POST:
					xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
					break;
				case HttpRequestMethod.DELETE:
				case HttpRequestMethod.CONNECT:
				case HttpRequestMethod.HEAD:
				case HttpRequestMethod.OPTIONS:
				case HttpRequestMethod.TRACE:
				case HttpRequestMethod.PATCH:
				default:
					unsupportedOperation();
			}
			xhr.send(data);
		}

		Object.forEach(HttpRequestMethod, (k, v) => {
			ajax[k.toLowerCase()] = (arg) => {
				arg.method = v;
				ajax(arg);
			}
		});

		return ajax;
	})()

	return {
		// 屏幕效果
		screen: {
			/*
			 * 代码来源：https://www.zybuluo.com/EncyKe/note/254250
			 * 有修改
			 */
			toPageTop: function () {

				var toTop = getScrollTop();

				// 设置计时器，50毫秒间隔；
				var toTopTimer = setInterval(function () {

					// 设置速度，用等式而不用具体数值是为了产生缓动效果；
					toTop -= Math.ceil(toTop / 5);

					// 作差，产生缓动效果；
					doc.documentElement.scrollTop = doc.body.scrollTop = toTop;
					// 判断是否抵达顶部，若是，停止计时器；
					if (toTop <= 0) {
						clearInterval(toTopTimer);
					};
				}, 50);
			},
			getScrollTop: getScrollTop,
			stopOnScreen: (function () {

				var elements = {};
				elements.id = [];

				function getOffsetTop(o) {

					var top = 0;
					var offsetParent = o;

					while (offsetParent != null && offsetParent != document.body) {

						top += offsetParent.offsetTop;
						offsetParent = offsetParent.offsetParent;
					}

					return top;
				}
				window.onscroll = function () {
					var top = getScrollTop();
					var id = elements.id;
					for (let i = 0, len = id.length; i < len; i++) {
						let element = elements[id[i]];

						top = top - element.top;
						if (top > 0) {
							element.element.style.top = top + "px";
						} else {
							element.element.style.top = 0 + "px";
						}
					}
				};

				return function (id, top) {
					if (!elements[id]) {
						elements[id] = {
							top: top,
							element: $dom.getNode(id)
						};
						elements.id.push(id);
					}
				};
			})()
		},
		// 公共模块
		common: (function () {

			return {
				cutTitle: function (str, max, cut) {
					if (max < str.length) {
						str = str.slice(0, cut) + " ......";
					}
					return str;
				},
				forwardTo: function (url) {
					location.href = url;
				},
				newWindow: newWindow,
				pathArray: function () {
					var path = (doc.location.href).split('/');
					var arr = path[path.length - 1].split('.');
					return arr;
				},
				/*
				 * showPage 只负责显示页码，
				 * 具体的现实样式交由外部 CSS 来自行实现
				 */
				showPage: (function () {
					function showLoop(pages, changeId, start, now, end) {
						if (end === 0) {
							pages.push('<span id="nowpage" title="当前页">1</span>');
						} else {
							for (; start < end; start++) {
								if (start !== now) {
									pages.push('<a class="showpage" href="javascript:front.onChange.page(' + changeId + ',' + start +
										')" title="前往第 ' + (start + 1) + ' 页">' + (start + 1) + '</a>');
								} else {
									pages.push('<span id="nowpage" title="当前页">' + (now + 1) + '</span>');
								}
							}
						}
					}
					return function (changeId, now, total) {
						var pages = [];
						/*
						 * total <= 10 一口气全部显示，且没随机页面跳转
						 * 
						 * 1 2 3 4 5 6 7 8 9 10
						 */
						if (total < 10) {
							showLoop(pages, changeId, 0, now, total + 1);
						} else {
							/*
							 * total > 10 && now < 5
							 * 
							 * 1 2 3 4 5 ... >> >|
							 */
							if (now < 5) {
								showLoop(pages, changeId, 0, now, 5);
								pages.push('...');
								pages.push('<a id="nextpage" href="javascript:front.onChange.page(' + changeId + ',' + (now + 1) +
									')" title="前往下一页（' + (now + 2) + '）">&gt&gt</a>');
								pages.push('<a id="toend" href="javascript:front.onChange.page(' + changeId + ',' + total +
									')" title="前往最终页（' + (total + 1) + '）">&gt|</a>');
							}
							/*
							 * total > 10 && now > total - 5
							 * 
							 * |< << ... 19 20 21 22 23
							 */
							else if (total - 5 < now) {
								pages.push('<a id="tofirst" href="javascript:front.onChange.page(' + changeId +
									',0)" title="前往第一页">|&lt;</a>');
								pages.push('<a id="prevpage" href="javascript:front.onChange.page(' + changeId + ',' + (now - 1) +
									')" title="前往上一页（' + now + '）">&lt;&lt;</a>');
								pages.push('...');
								showLoop(pages, changeId, total - 4, now, total + 1);
							}
							/*
							 * total > 10 && now < total - 5
							 * 
							 * |< << ... 13 14 15 16 17 ... >> >|
							 */
							else {
								pages.push('<a href="javascript:front.onChange.page(' + changeId + ',0)" title="前往第一页">|&lt;</a>');
								pages.push('<a href="javascript:front.onChange.page(' + changeId + ',' + (now - 1) +
									')" title="前往上一页（' + now + '）">&lt;&lt;</a>');
								pages.push('...');
								showLoop(pages, changeId, now - 2, now, now + 3);
								pages.push('...');
								pages.push('<a id="nextpage" href="javascript:front.onChange.page(' + changeId + ',' + (now + 1) +
									')" title="前往下一页（' + (now + 2) + '）">&gt;&gt;</a>');
								pages.push('<a id="toend" href="javascript:front.onChange.page(' + changeId + ',' + total +
									')" title="前往最终页（' + (total + 1) + '）">&gt|</a>');
							}
						}
						return pages.join('');
					};
				})()
			};
		})(),
		// dom 模块
		dom: $dom,
		form: {
			reset: function () {

			},
			submit: function (url, checkObj, before, after) {

				var data = {},
					hasCheckObj = !!checkObj;
				// 前切片
				before(data);

				if (hasCheckObj) {

					var beforeChk = checkObj.before;
					if (!!beforeChk) {
						var result = beforeChk(data);
						// 当返回 false 的时候，直接退出博客提交
						if (!result) return;
					}
				}

				__ajax.post({
					url: url,
					data: data,
					parse: true,
					success: function (req) {
						// 后切片
						if (req.success && hasCheckObj) {
							if (after) after(req);
						}
						// 后切片
						var afterChk = checkObj.after;
						if (!!afterChk) {
							afterChk(req, data);
						}
					}
				});
			}
		},
		// ajax 模块
		ajax: __ajax,
		animation: (function () {

			var TO_TOP = 0,
				TO_RIGHT = 1,
				TO_BOTTOM = 2,
				TO_LEFT = 3;

			function funcInterval(speed, target, inner, direction, callback) {

				return setInterval(function () {
					switch (direction) {
						case 0:
							if (target.scrollTop <= 0)
								target.scrollTop = inner[1].offsetHeight;
							else {
								target.scrollTop--;
							}
							break;
						case 1:
							if (target.scrollLeft === 0)
								target.scrollLeft = inner[1].offsetWidth;
							else {
								target.scrollLeft--;
							}
							break;
						case 2:
							if (inner[1].offsetHeight - target.scrollTop <= 0)
								target.scrollTop -= inner[0].offsetHeight;
							else {
								target.scrollTop++;
							}
							break;
						case 3:
							if (inner[1].offsetWidth - target.scrollLeft <= 0)
								target.scrollLeft -= inner[0].offsetWidth;
							else {
								target.scrollLeft++;
							}
							break;
						default:
							throw new Error("错误的方向");
					}

					if (callback) callback();

				}, speed);
			}

			function setting(node, width, height, tag, callback) {
				node.style.width = width + "px";
				node.style.height = height + "px";
				node.style.overflow = node.style.overflowX = node.style.overflowY = "hidden";

				var target = node.getElementsByTagName("div")[0];
				if (!target)
					return;
				var innerTag = target.getElementsByTagName(tag);
				if (!innerTag)
					return;

				callback(target, innerTag);
			}

			/*
			 * marquee 模块
			 * JS Marquee Version 1.2(Javascript实现Marquee的效果,实现无缝/有间隙的二维滚动效果)
			 * Author : ChenReal Email : chenreal@21cn.com Date : 2007-10-22 
			 * 有修改
			 * 
			 * id : 滚动目标的 ID
			 * tag : 滚动目标的标签
			 * width : 滚动范围的宽度
			 * height : 滚动范围的高度
			 * direction : 滚动方向：0:下->上，1:左->右，2:上->下，3:右->左
			 * speed : 滚动速度，0最快，无最慢
			 * space : 滚动时是否有间隙：true - 有间隙；false - 无间隙
			 * stop : 鼠标覆盖时是否停止：true - 停止；false - 不停止
			 */
			function marquee(id, tag, width, height, direction, speed, space, stop) {

				var intervalID;
				var target = $dom.getNode(id);

				setting(target, width, height, tag, function (innerDiv, innerTag) {

					innerDiv.innerHTML += innerDiv.innerHTML;
					switch (direction) {
						case 0:
							innerDiv.style.width = width + "px";
							innerTag[0].style.width = width + "px";
							innerTag[1].style.width = width + "px";
							if (height >= innerTag[1].offsetHeight || space) {
								innerTag[0].style.paddingBottom = height + "px";
								// innerTag[1].style.paddingBottom = height + "px";
							}
							break;
						case 1:
							innerDiv.noWrap = true;
							if (width >= innerTag[1].offsetWidth || space) {
								innerTag[0].style.paddingLeft = width + "px";
								innerTag[1].style.paddingLeft = width + "px";
							}
							break;
						case 2:
							innerDiv.style.width = width + "px";
							innerTag[0].style.width = width + "px";
							innerTag[1].style.width = width + "px";
							if (height >= innerTag[1].offsetHeight || space) {
								innerTag[0].style.paddingTop = height + "px";
								innerTag[1].style.paddingTop = height + "px";
							}
							break;
						case 3:
							innerDiv.noWrap = true;
							if (width >= innerTag[1].offsetWidth || space) {
								innerTag[0].style.paddingLeft = width + "px";
								innerTag[1].style.paddingLeft = width + "px";
							}
							break;
						default:
							var e = new Error("错误的方向");
							alert(direction);
							alert(e.stack);
							throw e;
					}

					if (stop) {
						target.onmouseover = function () {
							clearInterval(intervalID);
						};
						target.onmouseout = function () {
							intervalID = funcInterval(speed, target, innerTag, direction);
						};
					}
					intervalID = funcInterval(speed, target, innerTag, direction);
				});
			}

			function addButtons(node, elements, size, onMouseOverCallback, onMouseOutCallback) {

				// 创建按钮
				var buttons = $dom.create("ul", {
					className: 'slide_buttons'
				}),
					btnArray = [];

				// 添加 button 和 隐藏对象
				for (var i = 0; i < size; i++) {
					(function (element, index) {
						var li = $dom.create("li", {
							onmouseover: function () {
								onMouseOverCallback(index);
							},
							onmouseout: onMouseOutCallback,
							innerHTML: index + 1, // 显示编号
							className: "slice_button_" + ((i === 0) ? "on" : "out") // 设置初始状态
						});

						buttons.appendChild(li);
						btnArray.push(li);
					})(elements[i], i);
				}

				node.appendChild(buttons);

				return btnArray;
			}

			function changeButton(before, after) {
				before.className = "slice_button_out";
				after.className = "slice_button_on";
			}

			function slideshow(node, childTagName, interval, width, height, data) {

				var elements, intervalID, index = 0,
					size, buttons, _target;

				// 获得要进行移动的节点对象
				setting(node, width, height, "div", function (target, innerTag) {
					elements = target.getElementsByTagName(childTagName);
					size = elements.length;

					// 为整个 div 添加 onclick 事件，这样就不用为每个元素单独添加 onclick 事件了
					target.onclick = function () {
						var link = data[index].link;
						newWindow(link.url, elements[index].name, link.features);
					};

					_target = target;
				});

				function changeAltTitle() {
					_target.title = elements[index].name;
				}

				return {
					change: function () {

						function runChange() {
							intervalID = setInterval(function () {
								var before = index++;
								elements[before].style.display = "none";
								if (index === size) {
									index = 0;
								}
								elements[index].style.display = "block";
								changeButton(buttons[before], buttons[index]);
								changeAltTitle();
							}, interval);
						}

						buttons = addButtons(node, elements, size, function (i) {
							clearInterval(intervalID);
							elements[index].style.display = "none";
							elements[i].style.display = "block";
							changeButton(buttons[index], buttons[i]);
							changeAltTitle();
							index = i;
						}, runChange);
						runChange();
					},
					move: function (direction, speed) {

						function runMove() {
						}

						buttons = addButtons(node, elements, size, function () {
							clearInterval(intervalID);
						}, runMove);
						runMove();
					},
					fadeInOut: function (speed) {

						for (var i = 0; i < size; i++) {
							elements[i].zIndex = i;
							var style = elements[i].style;
							style.position = "absolute";
							style.top = "0px";
							style.left = "0px";
							style.opacity = 0;
						}
						elements[0].style.opacity = 1;

						function changeImg(before, now) {
							var opacityInterval = setInterval(function () {
								if (now.style.opacity > 1) {
									clearInterval(opacityInterval);
								}
								before.style.opacity -= 0.2;
								// 因为 style 全是字符串形式，用+会变成字符串连接，数字<->字符串之间的来回变换太烦，
								// 所以这里为了偷懒，直接用减法语法糖计算
								now.style.opacity -= -0.2;
							}, speed);

						}

						function runFadeInOut() {
							intervalID = setInterval(function () {
								var before = index++;
								elements[before].style.opacity = 1;
								if (index === size) {
									index = 0;
								}
								elements[index].style.opacity = 0;

								changeButton(buttons[before], buttons[index]);
								changeImg(elements[before], elements[index]);
								changeAltTitle();

							}, interval);
						}

						buttons = addButtons(node, elements, size, function (i) {
							changeImg(elements[index], elements[i]);
							changeButton(buttons[index], buttons[i]);
							index = i;
							clearInterval(intervalID);
							changeAltTitle();
						}, runFadeInOut);
						runFadeInOut();
					},
					/*
					 * firstColor：第一种颜色
					 * secondColor：第二种颜色
					 * rCnt：行
					 * cCnt：列
					 */
					chessboard: function (firstColor, secondColor, rCnt, cCnt, speed) {

						function runChessboard() { }

						buttons = addButtons(node, elements, size, function () {

						}, runChessboard);
						runChessboard();
					}
				};
			}

			return {
				TO_TOP: TO_TOP,
				TO_LEFT: TO_LEFT,
				TO_RIGHT: TO_RIGHT,
				TO_BOTTOM: TO_BOTTOM,
				marquee: marquee,
				slideshow: slideshow
			};
		})(),
		// Image 模块
		Image: (function () {

			function close() {
				album.removeChild(img);
				body.removeChild(album);
				body.removeChild(background);
			}

			var album = document.createElement('div');
			album.zIndex = "2";
			album.style.position = "absolute";
			var img = document.createElement('img');
			album.onclick = Function.EMPTY_BODY;
			var background = document.createElement('div');
			background.onclick = close;
			var body = null;

			return {
				resize: function (obj) {
					obj.proto = {
						width: obj.width,
						height: obj.height
					};
					if (obj.width > 800) {
						obj.width = 800;
					}
				},
				protoSize: function (obj) {

					var scrollHeight = document.documentElement.scrollHeight; // 滚动条高度，也就是整个页面的高度

					if (body === null) {
						body = document.body;
						background.zIndex = "1";
						background.style.width = "100%";
						background.style.height = scrollHeight + "px";
						background.style.background = "#000000";
						background.style.opacity = "0.85";
						background.style.position = "absolute";
						background.style.left = "0px";
						background.style.top = "0px";
					}
					var innerHeight = window.innerHeight; // 页面可视高度
					var innerWidth = window.innerWidth; // 页面可视宽度
					var protoHeight = obj.proto.height; // 图像原始高度
					var protoWidth = obj.proto.width; // 图像原始宽度
					var top, left;
					if (protoHeight < innerHeight) { // 图像高度 < 页面可视高度
						// 页面正中显示
						top = (innerHeight - protoHeight) / 2;
					} else { // 图像高度>页面可视
						// 页面上下 padding 40px 显示
						top = 40;
						if (protoHeight > scrollHeight - 80) { // 图像高度 > 滚动条高度-80
							//  页面上下 padding 40px 显示，且页面高度加到适合图像高度
							background.style.height = (protoHeight + 80) + "px";
						}
						// 图像高度 < 滚动条高度-80不做任何处理
					}

					if (protoWidth < innerWidth) { // 图像宽度 < 页面可视宽度
						// 页面正中显示
						left = (innerWidth - protoWidth) / 2;
					} else {
						// 调节滚动条宽度
						background.style.width = protoWidth + "80px";
						left = 40;
					}

					album.style.top = Math.ceil(getScrollTop() + top) + "px";
					album.style.left = Math.ceil(left) + "px";

					img.src = obj.src;
					img.height = protoHeight;
					img.width = protoWidth;
					img.title = obj.title;

					body.appendChild(background);
					album.appendChild(img);
					body.appendChild(album);
				},
				closeAlbum: close
			};
		})(),
		Validator: (function () {

			var WHITE = {
				background: '#FFFFFF'
			},
				RED = {
					background: '#FF0000'
				};

			function validateBlur($$, index) {
				$$.css(index, WHITE);
			}

			function validateFocus($$, index) {
				$$.event(index, 'focus');
			}

			function validateOnError($$, index, message) {
				if (!message) return;

				$$.css(index, RED);
				alert(message);
				validateFocus($$, index);
			}

			return {
				onError: validateOnError,
				blur: validateBlur,
				focus: validateFocus
			};
		})()
	};
})();
