;
/**
 * 自定义函数集/类库
 * <p>
 * 主要实现两个功能 <br />
 * 1.针对 JS 自身的不足，补足相关功能、添加部分数据类型 <br />
 * 设计方针基本符合《 JavaScript 语言精粹 》中的要求 <br />
 * &nbsp;&nbsp;&nbsp;&nbsp;不同的是因为要设计一些数据类型（Map、Set、Interface、Collection、XmlWrapper）等<br />
 * &nbsp;&nbsp;&nbsp;&nbsp;所以保留了对 new 关键字的使用<br />
 * 2.简单化功能封装 <br />
 * &nbsp;&nbsp;&nbsp;&nbsp;包括对一些 JS 中不尽人意的地方进行了封装，或者重新写了一些相适用的方法 / 函数集合
 * </p>
 * 所有实现均为对 ECMAScript 的扩展，不包含任何 window、document 操作<br />
 * 适用浏览器
 * <ul>
 * <li>Firefox</li>
 * <li>Chrome</li>
 * <li>Opera</li>
 * <li>Edge</li>
 * </ul>
 */
;
(function () {

	// 因为 Coralian! 需要 JSON 的支持，所以预先查询是否已载入 JSON 库
	if (!JSON) {
		throw new Error("您当前的环境不支持 JSON\n请先加载 JSON 库：http://www.json.org/");
	}

	// 设置端点，默认为 服务端
	var side = typeof (global) === 'undefined', // side = true 客户端; side = false 服务端
		// 定义 全局变量 that，node 中等价于 global 浏览器中等价于 window
		that = null,
		// 将 eval 函数的指针赋值给本地局部变量（暂时不知道能干嘛，保留指针）
		n_eval = null;

	var NBSP = '&nbsp;';

	/*
	 * 因为服务端（Nodejs）和客户端（浏览器）有不一样的地方 所以这里分别为服务端和客户端添加载入 Coralian
	 * 库所必需的函数或者一些方便调试的函数 通过 try... catch 块来分别执行 服务端和 客户端所需要加载的内容
	 */
	if (side) {
		that = window; // 初始化客户端
		that.global = {};

		// 如果客户端没有实现 console.log 则用 window.alert 来代为实现相关功能
		if (!window.console) {
			that.cosole = {
				log: that.alert,
				err: that.alert,
				warn: that.alert
			};
		} else if (!window.console.log) {
			that.console.log = that.alert
		}
	} else {
		that = global;
		that.alert = function (msg) {
			if (msg.contains("error")) {
				console.log(new Error());
			}
			console.log(msg);
		};
	}
	n_eval = that.eval;
	// 屏蔽 eval 函数
	that.eval = function () {
		var e = new Error("Coralian! 禁止使用 eval 函数，如果要解析 JSON 字符串，请使用 JSON.parse 来实现。");
	};

	// 设置 JSON 的两个处理为局部变量
	var JSONparse = JSON.parse,
		JSONstringify = JSON.stringify;

	/* ////////////////// 根据 JS1.85 进行的功能扩展：START ////////////////// */
	var EMPTY_STRING = "";
	/*
	 * 本地化 Date.prototype.toJSON 操作 时间日期的返回值是 YYYY-MM-DD hh:mm:ss 形式
	 */
	var defineProperty = Object.defineProperty;
	var array_slice = Array.prototype.slice;

	Date.prototype.toJSON = function () {

		return this.getFullYear() + "-" + (this.getMonth() + 1) + "-" + this.getDate() + " " + this.getHours() + ":" +
			this.getMinutes() + ":" + this.getSeconds();
	};
	Date.clone = Date.prototype.clone = function (date) {
		date = date || this;
		if (!(date instanceof Date)) {
			errorCast(date, Date);
		}
		var another = new Date(date.getTime());
		return another;
	};
	/*
	 * 这些功能在 JS1.85 中已经包含 但因为需要兼容其他未实现 JS1.85 的浏览器 所有自定义代码均来自 MDN https://developer.mozilla.org/
	 */
	/* ==================== Error 的扩展 ==================== */
	function debugError(e) {
		alert('message:' + e.message);
		alert('stack:' + e.stack);
		throw e;
	}

	function errorCast(obj, type) {

		if ('function' !== typeof type) {
			errorCast(type, Function);
		}

		var error = new TypeError();
		console.error(error.message);
		console.error(error.stack);
		error = typeOf(obj) + " 类型的数据无法转变为 " + type.getName() + "。";
		debugError(error);
	}
	Error.errorCast = errorCast;

	function indexOutOfBounds(given, want) {
		if (!isNumber(given)) {
			errorCast(given, Number);
		}
		if (!isNumber(want)) {
			errorCast(want, Number);
		}
		if (want < given) {
			debugError(new Error("请求的下标 " + given + " 超过了上限 " + want));
		} else {
			debugError(new Error("请求的下标 " + given + " 没有达到下限 " + want));
		}
	}
	Error.indexOutOfBounds = indexOutOfBounds;

	function noReference(msg) {
		var error = new ReferenceError();
		error.message = msg || "当前引用错误。";
		debugError(error);
	}
	Error.noReference = noReference;

	function unsupportedType(type) {
		var error = new TypeError();
		error.message = typeOf(type) + "类型的数据不被当前操作所支持。";
		debugError(error);
	}
	Error.unsupportedType = unsupportedType;

	function unsupportedOperation(msg) {
		msg += " 不被支持的操作";
		debugError(new Error(msg));
	}
	Error.unsupportedOperation = unsupportedOperation;

	function noSuchMethod(name) {
		debugError(new Error("方法 " + name + " 不存在。"));
	}
	Error.noSuchMethod = noSuchMethod;

	function noSuchProperty(name) {
		debugError(new Error("属性 " + name + " 不存在。"));
	}
	Error.noSuchProperty = noSuchProperty;

	function errorStatement() {
		debugError(new Error("语法错误，或被执行的逻辑不正确。"));
	}
	Error.errorStatement = errorStatement;

	function illegalArguments(msg) {
		msg = msg || '函数参数不正确';
		debugError(msg);
	}
	Error.illegalArguments = illegalArguments;
	/* ==================== Error 的扩展 ==================== */

	var _isArray = (function () {
		if (!Array.isArray) {
			function isArray(arr) {
				return arr && (arr instanceof Array ||
					typeof arr === 'object' &&
					typeof arr.length === 'number' &&
					arr.propertyIsEnumerable('length'));
			}
			Array.isArray = isArray;
		}
		return Array.isArray;
	})();

	/* ==================== Object 的扩展 ==================== */
	if (typeof Object.keys !== 'function') {
		Object.keys = function (o) {
			if (o !== Object(o))
				throw new TypeError('Object.keys called on a non-object');
			let k = [];
			for (let p in o) {
				if (hasOwnProperty(o, p)) {
					k.push(p);
				}
			}
			return k.sort();
		};
	}
	Object.values = function (o) {
		if (o !== Object(o))
			throw new TypeError('Object.keys called on a non-object');
		var k = [];
		for (let p in o) {
			if (hasOwnProperty(o, p)) {
				k.push(o[p]);
			}
		}
		return k.sort();
	}
	// 因为 Coralian! 中已经留有很多地方使用到了 keyArray 函数
	// 所以为了保证整体的可用性 以及代码的简洁性，保留 keyArray 指针
	var keyArray = Object.keys;
	/**
	 * 用于得到数据类型
	 * 一般以 string 类型返回数据类型的值
	 * 数据类型判断不依照 JS 规则，而是以以下基准判断
	 * 'null' === typeOf(null)
	 * 'undefined' === typeOf(undefined)
	 * 'boolean' === typeOf(boolean)
	 * 'NaN' === typeOf(NaN) 仅用于 NaN ，如果提供的是 '12X' 则直接判断为字符串
	 * 'Infinity' === typeOf(Infinity) 仅用于 Infinity 和 -Infinity，其他可用字面量表达的数字判断为 number
	 * 'number' === typeOf(number)
	 * 'string' === typeOf(string)
	 * 'function' === typeOf(function)
	 * 'object' === typeOf(object)
	 * 'array' === typeOf(array)
	 * 'regexp' === typeOf(regexp)
	 * 
	 * String、Number、Boolean 这三种可以被包装的对象也当成字面量来进行判断，而不返回 object
	 * 即
	 * 'string' === typeOf(new String());
	 * 'number' === typeOf(new Number());
	 * 'boolean' === typeOf(new Boolean());
	 * 
	 * 但不会判断包括JS 内置的 Date 类型，以及 XmllWrapper 在内的 Coralian 自定义数据类型，这些类型都将被辨认为 object
	 * 
	 * @returns
	 */
	function typeOf(object) {
		var result;

		if (object === null) {
			result = 'null';
		} else if (_isArray(object)) {
			result = 'array';
		} else if (object !== object) {
			result = 'NaN';
		} else if (object === Infinity || object === -Infinity) {
			result = 'Infinity';
		} else if (object instanceof RegExp) {
			result = 'regexp';
		} else if (object instanceof Number) { // new Number
			result = 'number';
		} else if (object instanceof Boolean) { // new Boolean
			result = 'boolean';
		} else if (object instanceof String) { // new String
			result = 'string';
		} else {
			result = typeof object;
			if (result === 'number' && isNaN(object)) { // 以防有漏网之鱼
				result = 'NaN';
			}
		}

		return result;
	}
	// 将 typeOf 函数分别添加到 Object 对象和 全局对象
	Object.typeOf = typeOf;
	that.typeOf = typeOf;

	/*
	 * 第一个参数是需要被比较的 Object 对象
	 * 从第二个参数开始是一个可变长参数，可以传入任意多字符串参数表达数据类型，
	 * 只要其中一个和被比较对象的结果一致，则返回 ture 当全部不符合的时候，返回 false 。
	 */
	function typeIs(object, types) {

		var type = typeOf(object);
		var _types = types;

		if (arguments.length === 2 && typeof types === 'string') {
			return type === types;
		} else if (!_isArray(types) && arguments.length > 2) {
			_types = Array.prototype.slice.call(arguments, 1);
		}

		return _types.has(type);
	}
	Object.typeIs = typeIs;
	that.typeIs = typeIs;

	function instanceTo(instance, type) {

		var prototype = type.prototype;

		if (prototype) {
			instance.__proto__ = prototype;
			instance.constructor = prototype.constructor;
		}
	}
	Object.instanceTo = instanceTo;

	function hashCode(element) {
		var value;
		try {
			value = JSONstringify(element);
		} catch (e) {
			value = element.toString();
		}

		var hashcode = 0;
		for (i = 0, len = value.length; i < len; i++) {
			char = value.charCodeAt(i);
			hashcode = ((hashcode << 5) - hashcode) + char;
			hashcode &= hashcode;
		}

		return hashcode;
	}
	Object.hashCode = hashCode;

	function equals(arg1, arg2) {

		if (arg1 === undefined) {
			if (arg2 === undefined) return true;
			else return false;
		}

		if (arg1 === null) {
			if (arg2 === null) return true;
			else return false;
		}

		if (arg2 === undefined) {
			arg2 = this;
		}
		if (arg1 === arg2) { // 值相同、数据类型相同，或者引用的是同一块内存
			return true;
		} else if (arg1 == arg2) { // 值相同、数据类型不同
			return false;
		}
		if (typeIs(arg1, 'function') && typeIs(arg2, 'function'))
			return arg1.toSource() === arg2.toSource();

		var isArray1 = _isArray(arg1),
			isArray2 = _isArray(arg2);
		if (isArray1 && isArray2) {
			if (arg1.length !== arg2.length) return false;

			for (let i = 0, len = arg1.length; i < len; i++) {
				if (!arg1[i].equals(arg2[i])) return false;
			}
			return true;
		} else if (!isArray1 && !isArray2) {
			// 提取对象的 KEY 为一个数组
			var key1 = keyArray(arg1),
				key2 = keyArray(arg2);
			if (key1.length === key2.length) {
				// 直接转化为字符串判断两个 KEY 的内容是否一致
				if (key1.toString() === key2.toString()) {
					for (let i = 0, len = key1.length; i < len; i++) {
						// 递归
						if (!equals(arg1[key1[i]], arg2[key2[i]])) return false;
					}
					return true;
				}
			}
		}
		return false;
	}
	Object.prototype.equals = equals;
	that.equals = equals;

	function objectIsEmpty(arg) {
		arg = arg || this;

		for (let k in arg) {
			if (hasOwnProperty(arg, k)) return false;

		}
		return true;
	};
	Object.prototype.isEmpty = objectIsEmpty;
	Object.isEmpty = function (obj) {
		if (obj === null || obj === undefined) return true;
		return objectIsEmpty(obj);
	};

	/*
	 * [ERR:20170310] 将 nodejs 更新至 v6.10.0 之后有些时候会出现 hasOwnProperty 错误。
	 * 错误信息：TypeError: hasOwnProperty is not a function
	 * 原因未知。
	 * 所以暂时用这种方式来解决问题
	 */
	function hasOwnProperty(obj, keyName) {
		return Object.prototype.hasOwnProperty.call(obj, keyName);
	}

	function addAll(from, to) {

		if (!typeIs(from, 'object', 'array')) unsupportedType(from);
		if (!typeIs(to, 'object', 'array')) unsupportedType(to);

		for (let key in from) {
			if (hasOwnProperty(from, key)) { // [ERR:20170310] 说的错误就是它！
				let now = from[key];
				if (typeIs(now, 'object')) {
					let _to = {};
					addAll(now, _to);
					to[key] = _to;
				} else {
					to[key] = from[key];
				}
			}
		}
	}
	Object.addAll = addAll;

	(function () {

		function newInstance(type, args) {

			var obj = {};
			var ret = type.apply(obj, args);
			var instance = typeIs(ret, 'object') ? ret : obj;

			return instanceTo(instance, type);
		}

		var FUNCTION_MARK = 'function ',
			ARG_MARK = 'arg';
		var BEFOR_BRACKET = '(',
			AFTER_BRACKET = ');';

		function getFunctionDefine(name, count) {
			var _d = [];
			for (let i = 0; i < count; i++) {
				_d.push(ARG_MARK + i);
			}
			return FUNCTION_MARK + name + BEFOR_BRACKET + _d.join() + AFTER_BRACKET;
		}

		function getDefine(func) {
			func = func || this;
			return getFunctionDefine(func.getName(), func.count);
		}

		Function.prototype.getDefine = getDefine;
		Function.getDefine = getDefine;

		function Constructor(type, name, callback, isFunction) {

			// 参数个数
			var count = type.length,
				// 获得定义
				define = getFunctionDefine(name, count);

			this.getDefine = function () {
				return define;
			};
			this.parameterCount = function () {
				return count;
			};
			this.newInstance = function () {
				callback();
				if (!isFunction) {
					return newInstance(type, arguments);
				} else {
					return new type();
				}
			};
		}

		function Type(obj) {

			// 对象的数据类型
			var type = obj.constructor || Object,
				// 对象的原型
				prototype = obj.prototype || Object;

			// 是否是基本数据类型
			var isPrimitive = (type === String || type === Number || type === Boolean),
				// 是否是接口
				_isInterface = isInterface(obj),
				// 是否是数组
				isArray = _isArray(obj),
				isFunction = (type === Function),
				// 是否是字面量对象
				isLiteral = (type === Object || isPrimitive || isArray || type === RegExp);

			// 类型的名字
			var name = type.getName();

			function constructorCheck() {
				if (isLiteral || isPrimitive) {
					throw new Error("请使用字面量来构造对象");
				}
				if (_isInterface) {
					throw new Error("请使用 Coralian.util.Interface 类的 newInterface 方法来构造接口");
				}
			}

			this.getName = function () {
				return name;
			};
			this.getStatic = type.getStatic;
			this.isLiteral = function () {
				return isLiteral;
			};
			this.isArray = function () {
				return isArray;
			};
			this.isInterface = function () {
				return _isInterface;
			};
			this.isPrimitive = function () {
				return isPrimitive;
			};
			this.isFunction = function () {
				return isFunction;
			},
				// TODO 这两个到底是个什么还没想好
				//			this.invoke = function() {
				//				if (isFunction) {
				//					return type.apply(null, arguments);
				//				} else {
				//					unsupportedOperation(typeOf(type) + "不是函数");
				//				}
				//			},
				this.newInstance = function () {
					constructorCheck();
					if (!isFunction) {
						return newInstance(type);
					} else {
						return new type();
					}
				};
			this.getPrototype = function () {
				return prototype;
			};
			this.typeIs = function () {
				return typeIs(obj, arguments);
			};
			this.instanceOf = function (classType) {
				return type === classType || type instanceof classType;
			};
			this.getConstructor = function () {
				return new Constructor(type, name, constructorCheck, isFunction);
			};
		}
		/*
		 * 通过 Type 类来获得相关 type 的信息
		 */
		function getType() {
			return new Type(this);
		}
		Object.prototype.getType = getType;
		Object.getType = function () {
			return Type;
		}

		Object.defineProperty(Type, 'null', {
			value: 'null',
			writable: false
		});
		Object.defineProperty(Type, 'array', {
			value: 'array',
			writable: false
		});
		Object.defineProperty(Type, 'NaN', {
			value: 'NaN',
			writable: false
		});
		Object.defineProperty(Type, 'Infinity', {
			value: 'Infinity',
			writable: false
		});
		Object.defineProperty(Type, 'regexp', {
			value: 'regexp',
			writable: false
		});
		Object.defineProperty(Type, 'number', {
			value: 'number',
			writable: false
		});
		Object.defineProperty(Type, 'boolean', {
			value: 'boolean',
			writable: false
		});
		Object.defineProperty(Type, 'string', {
			value: 'string',
			writable: false
		});
		Object.defineProperty(Type, 'object', {
			value: 'object',
			writable: false
		});
	})();

	function forEach(obj, callback) {
		if (obj === null || obj === undefined) noReference();
		if (obj instanceof Map) {
			for (let [key, value] of obj.entries()) {
				callback(key, value);
			}
		} else {
			switch (typeOf(obj)) {
				case 'array':
					Array.forEach(obj, callback);
					break;
				case 'object':
					for (let k in obj) {
						if (hasOwnProperty(obj, k)) {
							let result = callback(k, obj[k]);
							if (false === result) break;
							if (true === result) continue;
						}
					}
					break;
				default:
					unsupportedType(obj);
			}
		}
	}
	Object.forEach = forEach;
	that.forEach = forEach;
	Object.mix = function () {
		let deepCpy = arguments[0],
			start = 1;
		if (!typeIs(deepCpy, 'boolean')) {
			deepCpy = false;
			start = 0;
		}

		let result = {};

		for (let i = start; i < arguments.length; i++) {
			let e = arguments[i];

			if (!typeIs(e, 'object')) unsupportedType(e);

			let el = deepCpy ? JSON.parse(JSON.stringify(e)) : e;

			addAll(el, result);
		}

		return result;
	}

	function Iterator(obj) {
		if (obj === null || obj === undefined) noReference();

		var isArray = _isArray(obj);
		if (!isArray && !typeIs(obj, 'object')) unsupportedType(obj);

		var keys = isArray ? obj : keyArray(obj);
		var index = 0,
			count = keys.length;

		this.hasNext = function () {
			return index < count;
		};
		this.next = function () {
			var key = keys[index++];
			if (isArray) {
				return key;
			} else {
				return obj[key];
			}
		};
		this.first = function () {
			return keys[0];
		};
		this.last = function () {
			return keys[count - 1];
		};
		this["goto"] = function (index) {
			if (index < 0) {
				indexOutOfBounds(index, 0);
			}
			if (index >= count) {
				indexOutOfBounds(index, count);
			}
		};
		this.forward = function (cnt) {
			var at = index + cnt;
			if (at < 0) {
				indexOutOfBounds(at, 0);
			}
			if (at >= count) {
				indexOutOfBounds(at, count);
			}
			index = at;
		};
	}

	function iterator(obj) {
		return new Iterator(obj);
	};

	function objectClone(obj) {

		if (obj === null || obj === undefined) return obj;

		if (obj !== obj) {
			return obj;
		}
		if (typeIs(obj, 'string', 'number', 'Infinity', 'boolean', 'regexp', 'function')) return obj;

		if (Array.isArray(obj)) {
			return Array.clone(obj);
		}

		var another = {};
		for (let key in obj) {
			if (hasOwnProperty(obj, key)) {
				another[key] = objectClone(obj[key]);
			}
		}

		// 将新建对象的原型链关联到旧对象上，保证两者看起来一致
		instanceTo(another, obj.getType());

		return another;
	};

	/* ==================== Object 的扩展 ==================== */
	/* ==================== Array 的扩展 ==================== */
	Array.removeEach = function (array, callback) {
		if (!typeIs(array, 'array')) unsupportedType(array);

		while ((node = array.shift()) !== undefined) {
			callback(node);
		}

	};
	Array.forEach = function (array, start, end, callback) {
		if (!typeIs(array, 'array')) unsupportedType(array);

		switch (arguments.length) {
			case 2:
				callback = start;
				start = 0;
				end = array.length;
				break;
			case 3:
				callback = end;
				end = array.length;
				break;
			default:
				break;
		}

		if (start < 0) indexOutOfBounds(start, 0);
		if (end > array.length) indexOutOfBounds(end, array.length);

		for (let i = start; i < end; i++) {
			let result = callback(i, array[i]);
			if (false === result) break;
			if (true === result) continue;
		}
	};
	Array.prototype.equals = function (arr1) {
		var arr2 = this;
		if (!(arr1 instanceof Array)) {
			errorCast(arr1, Array);
		}
		if (!(arr2 instanceof Array)) {
			errorCast(arr1, Array);
		}
		var len = arg1.length;
		if (len === arr2.length) {
			for (var i = 0; i < len; i++) {
				if (!arr[1].equals(arr2[1])) {
					return false;
				}
			}
			return true;
		}
		return false;
	};
	Array.prototype.isEmpty = function () {
		return this.length === 0;
	};
	Array.prototype.asObject = function () {
		var obj = {};
		for (var i = 0, len = this.length; i < len; i++) {
			obj[i] = this[i];
		}
		return obj;
	};
	Array.prototype.has = function (element) {
		for (let i = 0, len = this.length; i < len; i++) {
			if (element === this[i]) {
				return true;
			}
		}
		return false;
	};
	Array.prototype.find = function (element) {
		for (let i = 0, len = this.length; i < len; i++) {
			if (element === this[i]) {
				return i;
			}
		}
		return -1;
	};
	Array.prototype.last = function (cnt) {
		if (cnt === undefined || cnt === null) {
			cnt = 1;
		}
		if (!typeIs(cnt, 'number')) {
			unsupportedType(cnt);
		}
		if (cnt > 0) {
			return this[this.length - cnt];
		} else {
			return this[0 - cnt];
		}
	};

	Array.asArray = function (obj) {
		if (!obj) return [];
		return array_slice.call(obj);
	};

	if (!Array.of) {
		Array.of = function () {
			return array_slice.call(arguments);
		};
	}

	Array.clone = Array.prototype.clone = function (array) {

		array = array || this;

		var output = [];

		for (let i = 0, len = array.length; i < len; i++) {
			output.push(objectClone(array[i]));
		}

		return output;
	};

	if (!Array.from) {
		Array.from = (function () {
			var toStr = Object.prototype.toString;
			var isCallable = function (fn) {
				return typeof fn === 'function' || toStr.call(fn) === '[object Function]';
			};
			var toInteger = function (value) {
				var number = Number(value);
				if (isNaN(number)) {
					return 0;
				}
				if (number === 0 || !isFinite(number)) {
					return number;
				}
				return (number > 0 ? 1 : -1) * Math.floor(Math.abs(number));
			};
			var maxSafeInteger = Math.pow(2, 53) - 1;
			var toLength = function (value) {
				var len = toInteger(value);
				return Math.min(Math.max(len, 0), maxSafeInteger);
			};

			// The length property of the from method is 1.
			return function from(arrayLike /*, mapFn, thisArg */) {
				// 1. Let C be the this value.
				var C = this;

				// 2. Let items be ToObject(arrayLike).
				var items = Object(arrayLike);

				// 3. ReturnIfAbrupt(items).
				if (arrayLike == null) {
					throw new TypeError("Array.from requires an array-like object - not null or undefined");
				}

				// 4. If mapfn is undefined, then let mapping be false.
				var mapFn = arguments.length > 1 ? arguments[1] : void undefined;
				var T;
				if (typeof mapFn !== 'undefined') {
					// 5. else      
					// 5. a If IsCallable(mapfn) is false, throw a TypeError exception.
					if (!isCallable(mapFn)) {
						throw new TypeError('Array.from: when provided, the second argument must be a function');
					}

					// 5. b. If thisArg was supplied, let T be thisArg; else let T be undefined.
					if (arguments.length > 2) {
						T = arguments[2];
					}
				}

				// 10. Let lenValue be Get(items, "length").
				// 11. Let len be ToLength(lenValue).
				var len = toLength(items.length);

				// 13. If IsConstructor(C) is true, then
				// 13. a. Let A be the result of calling the [[Construct]] internal method of C with an argument list containing the single item len.
				// 14. a. Else, Let A be ArrayCreate(len).
				var A = isCallable(C) ? Object(new C(len)) : new Array(len);

				// 16. Let k be 0.
				var k = 0;
				// 17. Repeat, while k < len… (also steps a - h)
				var kValue;
				while (k < len) {
					kValue = items[k];
					if (mapFn) {
						A[k] = typeof T === 'undefined' ? mapFn(kValue, k) : mapFn.call(T, kValue, k);
					} else {
						A[k] = kValue;
					}
					k += 1;
				}
				// 18. Let putStatus be Put(A, "length", len, true).
				A.length = len;
				// 20. Return A.
				return A;
			};
		}());
	}
	/* ==================== Array 的扩展 ==================== */
	/* ==================== String 的扩展 ==================== */
	String.isEmpty = function (input) {

		if (input === null || input === undefined) return true;
		if (!typeIs(input, 'string')) unsupportedType(input);

		return input.isEmpty() || input.length === 0; // 后面一半仅限于前半段不知道什么时候会坏掉时用
	};
	String.prototype.equals = function (another) {

		if (!typeIs(another, 'string')) return false; // another 不是字符串

		if (this === another) return true;

		// 添加这段代码主要是因为 new String() !== "" 所造成的问题，所以得比较字符串内部的结构
		if (this.length === another.length) {
			for (let i = 0, len = this.length; i < len; i++) {
				if (this[i] !== anothre[i]) return false;
			}
		}

		return true;

	};
	if (!String.prototype.contains) {
		String.prototype.contains = function (str, startIndex) {
			return -1 !== String.prototype.indexOf.call(this, str, startIndex);
		};
	}

	if (!String.prototype.equalsIgnoreCase) {
		String.prototype.equalsIgnoreCase = function (another) {
			var string = this;
			if (string === another) return true;

			if ((another !== null && another !== undefined) || string.length === another.length) {
				return string.toUpperCase() === another.toUpperCase();
			}

			return false;
		}
	}
	if (!String.prototype.endsWith) {
		Object.defineProperty(String.prototype, 'endsWith', {
			enumerable: false,
			configurable: false,
			writable: false,
			value: function (searchString, position) {
				position = position || this.length;
				position = position - searchString.length;
				var lastIndex = this.lastIndexOf(searchString);
				return lastIndex !== -1 && lastIndex === position;
			}
		});
	}
	String.prototype.last = function () {
		return this[this.length - 1];
	};
	String.prototype.lastCode = function () {
		return this.charCodeAt(this.length - 1);
	};
	if (!String.prototype.startsWith) {
		Object.defineProperty(String.prototype, 'startsWith', {
			enumerable: false,
			configurable: false,
			writable: false,
			value: function (searchString, position) {
				position = position || 0;
				return this.indexOf(searchString, position) === position;
			}
		});
	}

	function trimZeroWidth(str) {
		return str.replace(/\uFEFF/, EMPTY_STRING);
	}

	if (!String.prototype.trim) {
		String.prototype.trim = function () {
			var str = this.replace(/^\s+|\s+$/gm, EMPTY_STRING);
			return trimZeroWidth(str);
		};
	}
	if (!String.prototype.trimLeft) {
		String.prototype.trimLeft = function () {
			var str = this.replace(/(^\s*)/g, EMPTY_STRING);
			return trimZeroWidth(str);
		};
	}
	if (!String.prototype.trimRight) {
		String.prototype.trimRight = function () {
			var str = this.replace(/(\s*$)/g, EMPTY_STRING);
			return trimZeroWidth(str);
		};
	}
	if (!String.prototype.isEmpty) {
		String.prototype.isEmpty = function () {
			return this.length === 0;
		};
	}
	if (!String.prototype.compareTo) {
		String.prototype.compareTo = function (another) {
			var len1 = this.length,
				len2 = another.length;
			var max = len1 - len2;
			var len = (max <= 0) ? len1 : len2;
			var this_charCodeAt = this.charCodeAt;
			var another_charCodeAt = another.charCodeAt;
			for (let i = 0; i < len; i++) {
				let as = this_charCodeAt(i) - another_charCodeAt(i);
				if (as !== 0) return as;
			}
			return max;
		};
	}
	if (!String.fromCodePoint) {
		(function () {
			var defineProperty = (function () {
				// IE 8 only supports `Object.defineProperty` on DOM elements
				try {
					var object = {};
					return Object.defineProperty(object, object, object) && Object.defineProperty;
				} catch (error) { }
			}());
			var stringFromCharCode = String.fromCharCode;
			var floor = Math.floor;

			function fromCodePoint() {
				var MAX_SIZE = 0x4000;
				var codeUnits = [];
				var highSurrogate;
				var lowSurrogate;
				var index = -1;
				var length = arguments.length;
				if (!length) {
					return '';
				}
				var result = '';
				while (++index < length) {
					let codePoint = Number(arguments[index]);
					if (!isFinite(codePoint) || // `NaN`, `+Infinity`, or
						// `-Infinity`
						codePoint < 0 || // not a valid Unicode code point
						codePoint > 0x10FFFF || // not a valid Unicode code point
						floor(codePoint) != codePoint // not an integer
					) {
						throw RangeError('Invalid code point: ' + codePoint);
					}
					if (codePoint <= 0xFFFF) { // BMP code point
						codeUnits.push(codePoint);
					} else { // Astral code point; split in surrogate halves
						// http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
						codePoint -= 0x10000;
						highSurrogate = (codePoint >> 10) + 0xD800;
						lowSurrogate = (codePoint % 0x400) + 0xDC00;
						codeUnits.push(highSurrogate, lowSurrogate);
					}
					if (index + 1 == length || codeUnits.length > MAX_SIZE) {
						result += stringFromCharCode.apply(null, codeUnits);
						codeUnits.length = 0;
					}
				}
				return result;
			};
			if (defineProperty) {
				defineProperty(String, 'fromCodePoint', {
					'value': fromCodePoint,
					'configurable': true,
					'writable': true
				});
			} else {
				String.fromCodePoint = fromCodePoint;
			}
		}());
	}
	if (!String.prototype.codePointAt) {
		(function () {
			'use strict'; // needed to support `apply`/`call` with
			// `undefined`/`null`
			var codePointAt = function (position) {
				if (this == null) {
					throw TypeError();
				}
				var string = String(this);
				var size = string.length;
				// `ToInteger`
				var index = position ? new Number(position) : 0;
				if (index != index) { // better `isNaN`
					index = 0;
				}
				// Account for out-of-bounds indices:
				if (index < 0 || index >= size) return undefined;

				// Get the first code unit
				var first = string.charCodeAt(index);
				var second;
				if ( // check if it’s the start of a surrogate pair
					first >= 0xD800 && first <= 0xDBFF && // high surrogate
					size > index + 1 // there is a next code unit
				) {
					second = string.charCodeAt(index + 1);
					if (second >= 0xDC00 && second <= 0xDFFF) { // low surrogate
						// http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
						return (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
					}
				}
				return first;
			};
			Object.defineProperty(String.prototype, 'codePointAt', {
				'value': codePointAt,
				'configurable': true,
				'writable': true
			});
		}());
	}

	const STRING_AS_ARRAY_REGX = /[\s\S]/gi;
	String.prototype.asArray = String.asArray = function (input) {
   		input = input || this;
		return input.match(STRING_AS_ARRAY_REGX);
	};
	Object.defineProperty(String, 'BLANK', {
		value: EMPTY_STRING,
		writable: false
	});
	/* ==================== String 的扩展 ==================== */
	/* ==================== Number 的扩展 ==================== */
	function isNumber(number, notation) {

		if (number !== number) return false;

		notation = notation || 10;

		if (notation === 16 && !((number).toString().toLowerCase().startsWith('0x'))) {
			number = '0x' + number;
		}

		return isFinite(number);

	}
	Number.isNumber = isNumber;

	Number.prototype.equals = Number.equals = function (num1, num2) {

		if (!isNumber(num1)) unsupportedType(num1);

		if (num2 !== 0) {
			num2 = num2 || this;
		}

		if (!isNumber(num2)) unsupportedType(num2);

		return num1 == num2;
	};

	function parseNumber(num, nag, callback) {
		if (num === null || num === undefined) noReference();
		if (nag !== undefined && !isNumber(nag)) errorCast(nag, Number);

		if (!isNaN(num)) {
			return callback(num, nag || 10);
		} else {
			return NaN;
		}
	}

	Object.defineProperty(Number, 'HEX', {
		value: 16,
		writable: false
	});
	Object.defineProperty(Number, 'DEC', {
		value: 10,
		writable: false
	});
	Object.defineProperty(Number, 'OCT', {
		value: 8,
		writable: false
	});
	Object.defineProperty(Number, 'BIN', {
		value: 2,
		writable: false
	});
	/* ==================== Number 的扩展 ==================== */
	/* ==================== Function 的扩展 ==================== */
	function getName(func) {

		func = func || this;

		var functionName = String.BLANK;
		var _name = func.name;
		if (_name !== undefined) {
			functionName = _name;
		} else {
			let arr = null,
				str = func.toString();
			if (str.charAt(0) === '[') {
				arr = str.match(/\[\w+\s*(\w+)\]/);
			} else {
				arr = str.match(/function\s*(\w+)/);
			}
			if (arr && arr.length === 2) {
				functionName = arr[1];
			}
		}

		return functionName;
	}
	Function.prototype.getName = getName;
	Function.getName = getName;

	function getStatic(func) {

		func = func || this;

		var statics = {};

		for (let k in func) {
			if (hasOwnProperty(func, k)) {
				statics[k] = func[k];
			}
		}
		return !statics.isEmpty() ? statics : undefined;
	}
	Function.getStatic = getStatic;
	Function.prototype.getStatic = getStatic;

	if (!Function.prototype.toSource) {
		Function.prototype.toSrouce = Function.prototype.toString;
	}

	function EMPTY_BODY() { }
	Object.defineProperty(Function, EMPTY_BODY.getName(), {
		value: EMPTY_BODY,
		writable: false
	});
	/* ==================== Map 的扩展 ==================== */
	if (!Map.prototype.isEmpty) {
		Map.prototype.isEmpty = function () {
			return map.size === 0;
		}
	}
	/* ==================== Map 的扩展 ==================== */

	/* ==================== Function 的扩展 ==================== */
	/* ////////////////// 根据 JS1.85 进行的功能扩展：END ////////////////// */
	/* ////////////////// Coralian 自定义数据类型 ：START ////////////////// */
	/* =============== Coralian.util.NumberUitl.addComma =============== */
	function addComma(num, split) {

		if (!isNumber(num)) errorCast(num, Number);
		if (split !== undefined && !isNumber(split)) errorCast(split, Number);

		split = parseInt(Math.pow(10, split || 3));

		var result = String.BLANK;

		while (true) {
			let part = num % split;
			num = parseInt(num / split);

			rersult = part + result;
			if (num !== 0) {
				result = ',' + result;
			} else {
				break;
			}
		}

		return result;

	}

	/* =============== Coralian.util.Calendar =============== */
	var _calendar = (function () {
		// 一天的毫秒数
		var ONE_DAY_MILLISECONDS = 24 * 3600 * 1000;
		// 默认对时间进行格式化
		var DEFAULT_DT_FORMAT = "YYYY-MM-DD hh:mm:ss",
			YEAR_REGEXP = /(Y+)/;
		// 中文的周几
		var LONG_CHINESE_WEEK = ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六'],
			SHORT_CHINESE_WEEK = ['周日', '周一', '周二', '周三', '周四', '周五', '周六'];
		// 地支
		var EARTHLY_BRANCH = ['子', '丑', '寅', '卯', '辰', '巳', '午', '未', '申', '酉', '戌', '亥'];
		// 天干
		var CELESTIAL_STEM = ['甲', '乙', '丙', '丁', '戊', '己', '庚', '辛', '壬', '癸'];
		// 生肖
		var CHINESE_ZODIAC = ['鼠', '牛', '虎', '兔', '龙', '蛇', '马', '羊', '猴', '鸡', '狗', '猪'];
		var SEXAGENARY_CYCLE_FIRST = 4; // 公元 4年 甲子年
		// 英文的周几
		var ENGLISH_WEEK = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
		// 中文的几月
		var CHINESE_MONTH = ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月'];
		// 英文的几月
		var ENGLISH_MONTH = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'Augst', 'September',
			'October', 'November', 'December'
		];
		// 一个月有几天
		var MONTH_DAYS = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
		var CHINESE_CALANER = ['zh', 'xq', 'yue', 'nian'],
			ONE_COUNT = ['M', 'D', 'h', 'm', 's'];
		var TEN = 10,
			TWELVE = 12,
			THIRTY = 30,
			SEVEN = 7,
			TWENTY_NINE = 29,
			THIRTH_ONE = 31;

		function formatTime(date, format) {
			if (!(date instanceof Date)) {
				errorCast(date, Date);
			}
			format = format || DEFAULT_DT_FORMAT;
			var tmpMonth = date.getMonth();
			var tmpDay = date.getDay();
			var o = {
				"M+": (tmpMonth + 1), // month
				"yue": CHINESE_MONTH[tmpMonth],
				"month": ENGLISH_MONTH[tmpMonth],
				"mon": ENGLISH_MONTH[tmpMonth].slice(0, 3),
				"D+": date.getDate(), // day
				"h+": date.getHours(), // hour
				"m+": date.getMinutes(), // minute
				"s+": date.getSeconds(), // second
				"ms": date.getMilliseconds(), // millisecond
				"zh": SHORT_CHINESE_WEEK[tmpDay],
				"xq": LONG_CHINESE_WEEK[tmpDay],
				"w": ENGLISH_WEEK[tmpDay].slice(0, 3),
				"week": ENGLISH_WEEK[tmpDay]
			};
			if (YEAR_REGEXP.test(format)) {
				format = format.replace(RegExp.$1, (date.getFullYear()).toString().substr(4 - RegExp.$1.length));
			}
			for (let k in o) {
				if (hasOwnProperty(o, k)) {
					let regExp = new RegExp("(" + k + ")");
					if (regExp.test(format)) {
						let val = o[k];
						if (CHINESE_CALANER.has(k)) {
							format = format.replace(regExp, val);
						} else {
							if (!ONE_COUNT.has(RegExp.$1) && val < 10) {
								val = "0" + val;
							}
							format = format.replace(regExp, val);
						}
					}
				}
			}
			return format;
		}

		function instanceTime(arg, format) {
			if (!arg) {
				return new Date();
			} else if (arg instanceof Date) {
				return arg;
			} else if (isNumber(arg)) {
				let _date = new Date();
				format = format || 8;
				switch (format) {
					case 0:
						_date.setFullYear(arg);
						break;
					case 1:
						_data.setYear(arg);
						break;
					case 2:
						_data.setMonth(parseInt(arg) - 1);
						break;
					case 4:
						_data.setDate(arg);
						break;
					case 6:
						_date.setYear(parseInt(arg / 10000));
						_date.setMonth(parseInt(arg / 100) % 100 - 1);
						_date.setDate(arg % 100);
						break;
					case 8:
						_date.setFullYear(parseInt(arg / 10000));
						_date.setMonth(parseInt(arg / 100) % 100 - 1);
						_date.setDate(arg % 100);
						break;
					case 16:
						_date.setTime(arg);
						break;
					default:
						unsupportedOperation('所选择的格式化参数不正确');
				}
				return _date;
			} else {
				unsupportedType(arg);
			}
		}

		function isLeapYear(year) {

			return (year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0));
		}

		function checkMonth(month) {

			if (!isNumber(month)) errorCast(month, Number);
			if (month < 1 || 12 < month) unsupportedOperation(month + " 不是一个合法的月份");
		}

		function checkYear(year) {
			if (!isNumber(year)) errorCast(year, Number);
		}

		function getMonthDays(year, month) {

			var _isLeapYear = isLeapYear(year);

			return (month === 1 && _isLeapYear) ? TWENTY_NINE : MONTH_DAYS[month];
		}

		var _calendar = {
			/**
			 * 得到一定时间内的日历（公历表示，农历暂时不考虑）
			 * 得到一个二维表格，每行为7 第一天之前留空，最后剩余留空
			 * 
			 * 示例效果如下：
			 * [ [null, null, 1, 2, 3, 4, 5],
			 * [6, 7, 8, 9, 10, 11, 12],
			 * [13, 14, 15, 16, 17, 18, 19],
			 * [20, 21, 22, 23, 24, 25, 26],
			 * [27, 28, 29, 30, null, null, null] ]
			 * 
			 * 本函数只有两种重载：
			 *  1 参数指定：year，month，得到 year 年 month 月的月历
			 *  2 无参，得到当前月的月历
			 */
			monthly: function (year, month) {

				var date = year;
				if (year instanceof Date) {
					month = year.getMonth();
					year = year.getFullYear();
				} else {
					if (year) {
						checkYear(year);
					}
					if (month) {
						checkMonth(month);
						month -= 1;
					}
					date = new Date();
					if (year) {
						date.setYear(year);
					}
					if (month) {
						date.setMonth(month);
					}
				}
				date.setDate(1); // 月历固定每个月1号
				var weekDay = date.getDay();

				var days = 0,
					result = [],
					monthDays = getMonthDays(year, month);

				while (true) {
					if (days >= monthDays) break;
					let week = [null, null, null, null, null, null, null];
					for (let i = weekDay; i < 7; i++) {
						if (++days <= monthDays) {
							week[i] = days;
						} else {
							break;
						}
					}
					weekDay = 0;
					result.push(week);
				}

				return result;
			},
			/*
			 * today 代表目标日期，如果没有这个参数默认取“今天” week
			 * 代表要取的下一个星期几，如果没有这个参数默认取下一个星期天
			 */
			nextWeekDay: function (week, today) {
				var now = new Date();
				if (today) {
					now.setDate(today);
				}
				week = 7 - (week || now.getDay());

				return new Date(now.getTime() + ONE_DAY_MILLISECONDS * week);
			},
			// 返回被格式化之后的当前时间
			getTime: function (time, format) {
				if (time && !typeIs(time, 'number')) {
					time = parseInt(time);
				}
				return formatTime(instanceTime(time, 16), format);
			},
			formatTime: formatTime,
			instanceTime: instanceTime,
			isLeapYear: function (year) {

				if (!isNumber(year)) {
					errorCast(year, Number);
				}

				return isLeapYear(year);
			},
			getMonthDays: function (year, month) {

				checkYear(year);
				if (isNumber(month)) checkMonth(month);

				return getMonthDays(year, month);
			},
			getWeekDay: function (time, format) {
				return formatShow(instanceTime(time, format || 16));
			},
			// 获得农历表示法
			ChineseCalendar: {
				getChineseZodiac: function (year) {
					checkYear(year);
					return CHINESE_ZODIAC[(year - CHINESE_SEXAGENARY_CYCLE_FIRST) % TWELVE];
				},
				getCelestialStem: function (year) {
					checkYear(year);
					var stem = CELESTIAL_STEM[(year - SEXAGENARY_CYCLE_FIRST) % TEN] +
						EARTHLY_BRANCH[(year - SEXAGENARY_CYCLE_FIRST) % TWELVE];

					return stem;
				},
				/*
				 * 农历年表示法暂时不支持
				 */
				getSpringFestival: function (year) {
					checkYear(year);
					unsupportedOperation('暂不支持农历年表示法');
				},
				getMonthlyCalendar: function (year) {
					checkYear(year);
					unsupportedOperation('暂不支持农历年表示法');

				}
			}
		};

		Object.defineProperty(_calendar, 'YYYY', {
			value: 0,
			writable: false
		});
		Object.defineProperty(_calendar, 'YY', {
			value: 1,
			writable: false
		});
		Object.defineProperty(_calendar, 'Month', {
			value: 2,
			writable: false
		});
		Object.defineProperty(_calendar, 'Date', {
			value: 4,
			writable: false
		});
		Object.defineProperty(_calendar, 'YYMMDD', {
			value: 6,
			writable: false
		});
		Object.defineProperty(_calendar, 'YYYYMMDD', {
			value: 8,
			writable: false
		});
		Object.defineProperty(_calendar, 'MILLISECOND', {
			value: 16,
			writable: false
		});
		Object.defineProperty(_calendar, 'ONE_DAY_MILLISECONDS', {
			value: ONE_DAY_MILLISECONDS,
			writable: false
		});
		Object.defineProperty(_calendar, 'CHINESE_MONTH', {
			value: CHINESE_MONTH,
			writable: false
		});
		Object.defineProperty(_calendar, 'ENGLISH_WEEK', {
			value: ENGLISH_WEEK,
			writable: false
		});
		Object.defineProperty(_calendar, 'SHORT_ENGLISH_WEEK', {
			value: function () {
				var ew = [];
				for (let i = 0, len = ENGLISH_WEEK.length; i < len; i++) {
					ew[i] = ENGLISH_WEEK[i].slice(0, 3);
				}
				return ew;
			},
			writable: false
		});
		Object.defineProperty(_calendar, 'LONG_CHINESE_WEEK', {
			value: LONG_CHINESE_WEEK,
			writable: false
		});
		Object.defineProperty(_calendar, 'SHORT_CHINESE_WEEK', {
			value: SHORT_CHINESE_WEEK,
			writable: false
		});
		Object.defineProperty(_calendar, 'ENGLISH_MONTH', {
			value: ENGLISH_MONTH,
			writable: false
		});
		Object.defineProperty(_calendar, 'SHORT_ENGLISH_MONTH', {
			value: function () {
				var em = [];
				for (let i = 0, len = ENGLISH_MONTH.length; i < len; i++) {
					em[i] = ENGLISH_MONTH[i].slice(0, 3);
				}
				return em;
			},
			writable: false
		});

		return _calendar;
	})();
	/* =============== Coralian.util.Calendar =============== */
	/* =============== Coralian.util.Interface =============== */
	function Interface(name, methods) {
		for (let i = 0, len = methods.length; i < len; i++) {
			let method = methods[i];
			if (!typeIs(method, 'string')) errorCast(method, String);
		}
		this.getName = function () {
			return name;
		};
		this.iterator = function () {
			return new Iterator(methods);
		};
	}
	/* =============== Coralian.util.Interface =============== */

	function isInterface(obj) {
		return obj instanceof Interface;
	}

	function setToGlobal(parent, pkg, obj) {

		let name = pkg[0];
		if (pkg.length > 1) {
			let sub = parent[name];
			if (!sub) {
				sub = parent[name] = {};
			}
			setToGlobal(sub, pkg.slice(1), obj);
		} else {
			parent[name] = obj;
		}
	}

	var DEFAULT_PREFIX = "${",
		DEFAULT_SURFIX = "}";

	function replaceElement(str, obj, prefix, surfix) {

		if (!typeIs(str, 'string')) errorCast(str, String);

		prefix = prefix || DEFAULT_PREFIX;
		surfix = surfix || DEFAULT_SURFIX;

		var ret = EMPTY_STRING,
			p1 = 0,
			p2 = 0;
		while (true) {
			p1 = str.indexOf(prefix, p2);
			if (p1 === -1) break;
			if (p1 > p2) {
				ret += str.substring(p2, p1);
			}
			p2 = str.indexOf(surfix, p1 + prefix.length);
			if (p2 === -1) break;
			let holder = str.substring(p1 + prefix.length, p2);
			let nests = holder.split('.');
			let val = obj;
			for (let i = 0, length = nests.length; i < length; i++) {
				val = val[nests[i]];
				if (!val) break;
			}
			if (null !== val && undefined !== val) {
				// 如果目标对象中不包含请求内容，则清空表达
				ret += val;
			}
			p2 = p2 + surfix.length;
		}
		if (p1 === -1) {
			ret += str.substring(p2, str.length);
		} else {
			if (p2 === -1) {
				ret += str.substring(p1, str.length);
			}
		}
		return ret;
	}

	function formatString(str, obj) {
		var arglen = arguments.length;
		if (arglen > 2) {
			for (let i = 1; i < arglen; i++) {
				str = str.replace(/\%s/, arguments[i]);
			}
		} else if (arglen === 2) {
			if (typeIs(obj, 'array')) {
				Object.forEach(obj, function (i, e) {
					str = str.replace(/\%s/, e);
				});
			} else {
				str = replaceElement(str, obj);
			}
		} else {
			unsupportedOperation('至少需要一个字符来进行替换');
		}

		return str;
	}

	const ONLY_FORMAT = "只能在%s中使该功能用";

	function browserOnly() {
		if (!side) throw new Error(formatString(ONLY_FORMAT, "浏览器"));
	}

	function serverOnly() {
		if (side) throw new Error(formatString(ONLY_FORMAT, "服务端"));
	}

	var constants = {};
	Object.defineProperty(constants, 'MimeType', {
		value : {
			// application
			JSON : 'application/json',
			PDF : 'application/pdf',
			JAVASCRIPT : 'application/javascript',
			OCTET_STREAM : 'application/octet-stream',
			DTD : 'application/xml-dtd',
			ZIP : 'application/zip',
			// text
			TEXT : 'text/plain',
			HTML : 'text/html',
			XML : 'text/xml',
			CSS : 'text/css',
			CSV : 'text/csv',
			// image
			GIF : 'image/gif', 
			PNG : 'image/png',
			JPG : 'image/jpeg', 
			BMP : 'image/bmp',
			WEBP : 'image/webp',
			ICON : 'image/x-icon',
			SVG : 'image/svg+xml'
		},
		writeable : false
	});
	Object.defineProperty(constants, "HttpRequestMethod", {
		value : {
			GET : 'GET',
			PUT : 'PUT',
			POST : 'POST',
			DELETE : 'DELETE',
			CONNECT : 'CONNECT',
			HEAD : 'HEAD',
			OPTIONS : 'OPTIONS',
			TRACE : 'TRACE',
			PATCH : 'PATCH'
		},
		writeable : false
	});
	Object.defineProperty(constants, "HttpStatusCode", {
		value : {
			"CONTINUE":100,
			"SWITCHING_PROTOCOLS":101,
			"PROCESSING":102,
			"OK":200,
			"CREATED":201,
			"ACCEPTED":202,
			"NON_AUTHORITATIVE_INFORMATION":203,
			"NO_CONTENT":204,
			"RESET_CONTENT":205,
			"PARTIAL_CONTENT":206,
			"MULTI_STATUS":207,
			"ALREADY_REPORTED":208,
			"IM_USED":226,
			"MULTIPLE_CHOICES":300,
			"MOVED_PERMANENTLY":301,
			"FOUND":302,
			"SEE_OTHER":303,
			"NOT_MODIFIED":304,
			"USE_PROXY":305,
			"TEMPORARY_REDIRECT":307,
			"PERMANENT_REDIRECT":308,
			"BAD_REQUEST":400,
			"UNAUTHORIZED":401,
			"PAYMENT_REQUIRED":402,
			"FORBIDDEN":403,
			"NOT_FOUND":404,
			"METHOD_NOT_ALLOWED":405,
			"NOT_ACCEPTABLE":406,
			"PROXY_AUTHENTICATION_REQUIRED":407,
			"REQUEST_TIMEOUT":408,
			"CONFLICT":409,
			"GONE":410,
			"LENGTH_REQUIRED":411,
			"PRECONDITION_FAILED":412,
			"PAYLOAD_TOO_LARGE":413,
			"URI_TOO_LONG":414,
			"UNSUPPORTED_MEDIA_TYPE":415,
			"RANGE_NOT_SATISFIABLE":416,
			"EXPECTATION_FAILED":417,
			"I_AM_A_TEAPOT":418,
			"MISDIRECTED_REQUEST":421,
			"UNPROCESSABLE_ENTITY":422,
			"LOCKED":423,
			"FAILED_DEPENDENCY":424,
			"UNORDERED_COLLECTION":425,
			"UPGRADE_REQUIRED":426,
			"PRECONDITION_REQUIRED":428,
			"TOO_MANY_REQUESTS":429,
			"REQUEST_HEADER_FIELDS_TOO_LARGE":431,
			"UNAVAILABLE_FOR_LEGAL_REASONS":451,
			"INTERNAL_SERVER_ERROR":500,
			"NOT_IMPLEMENTED":501,
			"BAD_GATEWAY":502,
			"SERVICE_UNAVAILABLE":503,
			"GATEWAY_TIMEOUT":504,
			"HTTP_VERSION_NOT_SUPPORTED":505,
			"VARIANT_ALSO_NEGOTIATES":506,
			"INSUFFICIENT_STORAGE":507,
			"LOOP_DETECTED":508,
			"BANDWIDTH_LIMIT_EXCEEDED":509,
			"NOT_EXTENDED":510,
			"NETWORK_AUTHENTICATION_REQUIRED":511
		},
		writeable : false
	});

	/* ////////////////// Coralian 自定义数据类型 ：END ////////////////// */
	// Coralian 自身，将 Coralian 赋值给 that，作为全局变量
	let Coralian = {
		ABOUT: 'Coralian',
		VERSION: '0.0.1',
		HREF: 'http://www.waygc.net',
		AUTHOR: 'hzwaygc@gmail.com',
		side: function () {
			return side;
		},
		/*
		 * 下面两个方法是用来描述作用环境的
		 * 如果判断执行环境非作用环境，则抛出异常。
		 */
		browserOnly: browserOnly,
		serverOnly: serverOnly,
		/*
		 * Coralian.logger 只实现最基本的日志记录功能，
		 * 以及对日志进行最基本的格式化功能
		 */
		logger: (function () {

			var INFO = 'INFO',
				ERROR = 'ERROR',
				WARN = 'WARN';

			var STRING_FORMAT = "[${date}] ${level} ${message}";
			var TIME_FORMAT = "YYYY-MM-DD hh:mm:ss.ms";

			function format(level, message) {
				return formatString(STRING_FORMAT, {
					date: _calendar.formatTime(new Date(), TIME_FORMAT),
					level: level,
					message: message
				});
			}

			return {
				log: function (str) {
					console.log(format(INFO, str));
				},
				err: function (str) {
					console.error(format(ERROR, str));
				},
				warn: function (str) {
					console.warn(format(WARN, str));
				}
			}
		})(),
		/*
		 * 这组函数主要是用来实现将对象设置到 global 中用
		 * 和计划的解耦客户-服务端的不同方式，自己调用一个函数就可以的目的有些不同
		 * 这里更希望能实现一个类似 Java Class.forName 这样的一个功能
		 */
		setToGlobal: function (name, obj) {

			if (obj === null || obj === undefined) noReference();
			if (!typeIs(name, 'string')) unsupportedType(name);
			if (name.isEmpty()) throw new Error("不能使用空字符串作为属性名");

			setToGlobal(that, name.split("."), obj);
		},
		getFromGlobal: function (name) {

			if (!typeIs(name, 'string')) unsupportedType(name);
			if (name.isEmpty()) throw new Error("不能使用空字符串作为属性名");

			name = name.split(".");

			var obj = that[name[0]];
			for (let i = 1, len = name.length; i < len; i++) {
				if (obj === null || obj === undefined) {
					noReference();
				}
				obj = obj[name[i]];
			}
			if (!obj) {
				noReference();
			}
			return obj;
		},
		exports: function (name, obj) {

			setExports(that, name, obj);
		},
		// EFS 的基础数据类型
		// 以及相关方法
		util: {
			MathUtil: {
				/*
				 * 代码来自：https://tool.lu/hexconvert/
				 * 有修改。
				 *
				 * 进制转换函数。
				 * 有三个参数：
				 *   第一个参数支持的数据类型包括字符串和数字，
				 *     当第一个参数的数据类型为数字（typeof === 'number'），
				 *       且没有提供第三个参数，则第二个参数的值作为输出参数处理。
				 *   第二个参数是输入进制基数，
				 *      如果不提供该参数，则判断输入的数字为10进制；
				 *      如果输入的字符串中含有该进制不认可的字符，则返回NaN；
				 *      如果hexConvert 不支持该进制，则抛出错误。
				 *   第三个参数是输出进制基数，
				 *      当第一个参数的数据类型为数字（typeof === 'number'），该参数无效，
				 *        以第二个参数作为输出进制基数进行处理
				 *      如果不提供该参数，则判断输入的数字为10进制；
				 *      如果hexConvert 不支持该进制，则抛出错误。
				 */
				hexConvert: (function () {
					const BIN = 2,
						SIX = 6,
						OCT = 8,
						DEC = 10,
						HEX = 16,
						TWENTY_SIX = 26,
						SIXTY_TWO = 62
					SIXTY_FOUR = 64;

					const BIN_START = '0b',
						OCT_START = '0o',
						HEX_START = '0x'

					const NaN_STR = 'NaN';
					const HEX_CHARS = {};
					HEX_CHARS[DEC] = '0123456789';
					HEX_CHARS[HEX] = HEX_CHARS[DEC] + 'ABCDEF';
					HEX_CHARS[TWENTY_SIX] = 'abcdefghijklmnopqrstuvwxyz';
					HEX_CHARS[SIXTY_FOUR] = (HEX_CHARS[DEC] + HEX_CHARS[TWENTY_SIX] + HEX_CHARS[TWENTY_SIX].toUpperCase()).replace(/[OolI]/g, '') + '!@$&#%';
					HEX_CHARS[SIXTY_TWO] = HEX_CHARS[SIXTY_FOUR].slice(2);

					// Returns a string representation of the given number for the given alphabet:
					function toAlphabet(num, alphabet) {
						var base = alphabet.length;
						var digits = [];	// these will be in reverse order since arrays are stacks

						// execute at least once, even if num is 0, since we should return the '0':
						do {
							digits.push(num % base);	// TODO handle negatives properly?
							num = Math.floor(num / base);
						} while (num > 0);

						var chars = [];
						while (digits.length) {
							chars.push(alphabet[digits.pop()]);
						}
						return chars.join('');
					};

					// Returns an integer representation of the given string for the given alphabet:
					function fromAlphabet(str, alphabet) {
						var base = alphabet.length;
						var pos = 0;
						var num = 0;

						for (let chr of str) {
							let at = alphabet.indexOf(chr);
							if (at === -1) return NaN; // 因为返回的类型都是数字，所以这里直接返回 NaN
							num += Math.pow(base, pos) * at;
							pos++;
						}

						return num;
					};

					function fromBase(str, base) {

						switch (base) {
							case BIN:
								str = BIN_START + str;
								return Number(str);
							case OCT:
								str = OCT_START + str;
								return Number(str);
							case DEC:
								return Number(str);
							case HEX:
								str = HEX_START + str;
								return Number(str);
							default:
								return fromAlphabet(str, HEX_CHARS[base]);
						}
					};

					function hexConvert(num, inBase = DEC, outBase = DEC) {

						inBase = Number(inBase);
						outBase = Number(outBase);

						if (inBase > HEX && !(inBase in HEX_CHARS)) illegalArguments("不支持的输入进制基数：" + inBase);
						if (outBase > HEX && !(outBase in HEX_CHARS)) illegalArguments("不支持的输出进制基数：" + outBase);

						let type = typeOf(num);
						switch (type) {
							case 'string':
								num = fromBase(num, inBase);
								if (Object.is(num, NaN)) return NaN_STR;
								break;
							case 'number':
								outBase = inBase;
								break;
							default:
								Error.illegalArguments("不被支持的参数类型：" + type);
						}

						let hexChars = (outBase < DEC) ? HEX_CHARS[DEC].slice(0, outBase) : HEX_CHARS[outBase];
						return toAlphabet(num, hexChars);
					}

					hexConvert.BIN = BIN;
					hexConvert.OCT = OCT;
					hexConvert.DEC = DEC;
					hexConvert.HEX = HEX;
					hexConvert.TWENTY_SIX = TWENTY_SIX;
					hexConvert.SIXTY_TWO = SIXTY_TWO;
					hexConvert.SIXTY_FOUR = SIXTY_FOUR;
					return hexConvert;
				})(),
				PrimeNumber: (function () {

					const PRIME_IN_50 = [3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47]; // 预置50以内的所有质数

					function getPrimeInInputStartsWithFrom(primes, from) {

						let output = [];

						for (let i = primes.length - 1; i >= 0; i--) {
							if (primes[i] < from) break;
							output.unshift(primes[i]);
						}

						if (from < 2) {
							output.unshift(2);
						}

						return output;
					}

					return {
						calcPrimeByInput: (from, to) => {

							// 错误检查
							if (!Number.isNumber(from)) Error.errorCast(from, Number);
							if (!Number.isNumber(to)) Error.errorCast(to, Number);
							if (from > to) Error.unsupportedOperate("开始数必须大于结束数");
							if (to < 0 || from < 0) Error.unsupportedOperate("开始数必须大于 0");

							let primes = PRIME_IN_50.slice();
							let output = getPrimeInInputStartsWithFrom(primes, from);

							IS_PRIME: for (let i = 53; i < to; i += 2) {

								for (let j = 0; j < primes.length; j++) {
									if (i % primes[j] === 0) continue IS_PRIME;
								}

								primes.push(i);
								if (i >= from) {
									output.push(i);
								}
							}

							return output;
						},
						isPrimeNumber: (input) => {

							if (!typeIs(input, 'number')) Error.errorCast(input, Number);

							let inputRoot = Math.sqrt(input);

							for (let i = 0; i < PRIME_IN_50.length; i++) {
								let now = PRIME_IN_50[i];
								if (now > inputRoot) return true;
								if (input % now === 0) return false;
							}

							for (let i = 51; i < inputRoot; i += 2) {
								if (input % i === 0) return false;
							}

							return true;
						}
					};
				})()
			},
			ReplaceHolder: (function () {

				var LOOP_REG_START = "#{",
					LOOP_REG_END = "#{/",
					LOOP_IN_START = "#:{";
				var LOOP_REG_START_L = LOOP_REG_START.length;

				function replaceLoop(str, objs, callback) {
					var result = [];
					while (true) {
						let index = str.indexOf(LOOP_REG_START);
						if (index > 0) {
							result.push(str.slice(0, index));
							str = str.slice(index);
						} else if (index < 0) {
							result.push(str);
							break;
						}
						let firstEnd = str.indexOf(DEFAULT_SURFIX),
							action = str.slice(LOOP_REG_START_L, firstEnd);
						var end = LOOP_REG_END + action + DEFAULT_SURFIX;
						let inStr = str.slice(firstEnd + 1, str.indexOf(end));
						let obj = objs,
							tmpAction = action.split(".");
						for (let i = 0, len = tmpAction.length; i < len; i++) {
							if (!obj) break;
							obj = obj[tmpAction[i]];
						}

						if (obj !== null && obj !== undefined) {
							let isArr = _isArray(obj);
							let keys = isArr ? obj : keyArray(obj);
							for (let i = 0, len = keys.length; i < len; i++) {
								let o = obj[isArr ? i : keys[i]];
								let loopStr = inStr; // 定义循环所使用的变量
								loopStr = loopStr.replace(new RegExp(LOOP_IN_START + "id}", "g"), i);
								loopStr = loopStr.replace(new RegExp(LOOP_IN_START + "key}", "g"), keys[i]);
								if ('string' === typeof o) {
									result.push(loopStr.replace(new RegExp(LOOP_IN_START + "text}", "g"), keys[i]));
								} else {
									// 递归查找子循环
									loopStr = replaceLoop(loopStr, o);
									// 替换内部元素
									loopStr = replaceElement(loopStr, o, LOOP_IN_START + action + ".");
									loopStr = callback(loopStr, o, action + ".");
									result.push(loopStr);
								}
							}
						}
						str = str.slice(str.indexOf(end) + end.length);
					}

					return result.join(String.BLANK);
				}

				return {
					replaceElement: replaceElement,
					replaceLoop: replaceLoop,
					/*
					 * html 替换器
					 */
					htmlEscape: function (str, aspect) {

						str = str.replace(/\&/g, "&amp;");

						var first, second, third, forth;
						if (aspect) {
							first = aspect.first;
							second = aspect.second;
							third = aspect.third;
							forth = aspect.forth;
						}

						str = (!!first) ? first(str) : str;

						str = str.replace(/  /ig, ' &nbsp;');
						str = str.replace(/\t/ig, '&nbsp;&nbsp;&nbsp;&nbsp;\uFEFF'); // 制表符这么写主要是为了满足表现形式和制表符相同（不会换行）

						str = (!!second) ? second(str) : str;

						str = str.replace(/</ig, '&lt;');
						str = str.replace(/>/ig, '&gt;');

						str = (!!third) ? third(str) : str;

						str = str.replace(/(\r\n|\n|\r)/ig, '<br />');

						str = (!!forth) ? forth(str) : str;

						return str;
					}
				};
			})(),
			/*
			 * Coralian.util.Calendar
			 * 对时间处理的类集合封装了获得历法计算、时间计算等函数 包含对 Calendar 的相关操作
			 */
			Calendar: _calendar,
			ObjectUtil: {
				hasOwnProperty: hasOwnProperty,
				override: function (obj, name, callback) {

					var proto = obj[name];

					obj[name] = callback(function () {
						return proto.apply(obj, arguments);
					});
				},
				argumentsToArray: function () {
					return Array.of(arguments);
				},
				iterator: iterator,
				clone: objectClone
			},
			// Coralian.util.StringUtil
			StringUtil: {
				normal: function (input) {
					return input.replace(/<(\/)?(b|i|strike|em|u|strong|sub|sup)>/ig, EMPTY_STRING);
				},
				weight: function (tag) {
					return "<" + tag + ">" + input + "</" + tag + ">";
				},
				queryString: function (obj) {

					if (typeIs(obj, 'string')) {
						return encodeURI(obj);
					}

					var string = [];
					for (var key in obj) {
						if (hasOwnProperty(obj, key)) {
							// 这里 encodeURIComponent 是因为 nodejs 端才这么写，其他服务器语言可能用不着
							// JSON.stringify 这句是因为可能存在包含对象类型的数据
							if (typeIs(obj[key], 'object')) {
								string.push(key + '=' + JSONstringify(encodeURIComponent(obj[key])));
							} else {
								string.push(key + '=' + encodeURIComponent(obj[key]));
							}
						}
					}
					return string.join('&');
				},
				firstToUpperCase: function (str) {
					if (!typeIs(str, 'string')) errorCast(str, String);
					return str.charAt(0).toUpperCase() + str.slice(1);
				},
				firstToLowserCase: function (str) {
					if (!typeIs(str, 'string')) errorCast(str, String);
					return str.charAt(0).toLowserCase() + str.slice(1);
				}
			},
			// Coralian.util.NumberUtil
			NumberUtil: {
				addBeforeZero: function (arg, length) {
					if (!typeIs(arg, 'number')) unsupportedType(arg);
					if (!typeIs(length, 'number')) unsupportedType(length);

					var r = [];
					for (let i = 0; i < length; i++) {
						r.push("0");
					}
					return r.join(EMPTY_STRING) + arg;
				},
				addComma: addComma,
				parseInt: function (num, nag) {
					return parseNumber(num, nag, parseInt);
				},
				parseFloat: function (num, nag) {
					return parseNumber(num, nag, parseFloat);
				}
			},
			CharUtil: (function () {

				function invaildCharacter(input) {
					throw new Error(input + ' 不是合法的字符');
				}

				var ZERO = 0x0030,
					NINE = 0x0039,
					UPPER_A = 0x0041,
					UPPER_Z = 0x005A,
					LOWER_A = 0x0061,
					LOWER_Z = 0x007A;
				var SPACE = [' ', '\t', '\r', '\n', '\r\n', '\u000B', '\f', '\u001C', '\u001D', '\u001E', '\u001F'];

				function __isNumber(number) {
					if (!isChar(input)) invaildCharacter(input);
					var code = number.charCodeAt(0);
					return code >= ZERO && code <= NINE;
				}

				function isAlphabet(input) {
					if (!isChar(input)) invaildCharacter(input);
					var code = letter.charCodeAt(0);
					return (code >= UPPER_A && code <= UPPER_Z) || (code >= LOWER_A && code <= LOWER_Z);
				}

				function isChar(str) {
					if (!typeIs(str, 'string')) errorCast(str, String);
					return 1 === str.length;
				}

				return {
					isChar: isChar,
					isNumber: __isNumber,
					isAlphabet: isAlphabet,
					isNumberOrAlphabet: function (arg) {
						return isNumber(arg) || isAlphabet(arg);
					},
					isASCII(input) {
						if (!isChar(input)) invaildCharacter(input);
						return input.charAodeAt(0) < 128;
					},
					isSpace: function (input) {
						if (!isChar(input)) invaildCharacter(input);
						return SPACE.has(input);
					},
					change: function (value, count) {
						if (!isNumber(count)) errorCast(count, Number);
						if (isChar(value)) {
							return String.fromCharCode(value.charCodeAt(0) + count);
						} else {
							unsupportedOperation(value + ' 不是合法的字符');
						}
					},
					compare: function (char1, char2) {
						if (!isChar(char1)) invaildCharacter(char1);
						if (!isChar(char2)) invaildCharacter(char2);
						return char1.charCodeAt(0) - char2.charCodeAt(0);
					},
					asUnicodeEncode: function (input) {
						var code;
						if (!isNumber(input)) {
							if (!isChar(input)) invaildCharacter(input);
							code = (input.charCodeAt(0)).toString(16);
						} else {
							code = (input).toString();
						}
						code = code.toUpperCase();
						switch (code.length) {
							case 1:
								code = "000" + code;
								break;
							case 2:
								code = "00" + code;
								break;
							case 3:
								code = "0" + code;
								break;
							default:
								break;
						}
						return "\\u" + code;
					},
					ZERO: ZERO,
					NINE: NINE,
					UPPER_A: UPPER_A,
					UPPER_Z: UPPER_Z,
					LOWER_A: LOWER_A,
					LOWER_Z: LOWER_Z
				};
			})(),
			Format: (function () {

				function formatNumber(number, type) {

					// 目前只实现保留两位小数
					number = (number).toString().split(".");

					var after = number[1];
					if (after && after.length > 2) {
						let last = parseInt(after[2]);
						after = parseInt(after.slice(0, 2));
						if (last > 4) {
							after += 1;
						}
					}
					number = number[0] + "." + (after || 0);

					return number;
				}

				var DEFAULT_FORMAT_TYPE = "#.00";

				return {
					formatNumber: formatNumber,
					formatFileSize: function (size, type) {
						var formated;
						type = type || DEFAULT_FORMAT_TYPE;
						if (size < 1048576) {
							formated = formatNumber(size / 1024, type) + " KB";
						} else if (size < 1073741824) {
							formated = formatNumber(size / 1048576, type) + " MB";
						} else if (size < 1099511627776) {
							formated = formatNumber(size / 1073741824, type) + " GB";
						} else {
							formated = formatNumber(size / 1099511627776, type) + " TB";
						}
						return formated;
					},
					formatString: formatString
				};
			})(),
			Random: (function () {
				var ALL_CASE = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
				var UPPER_CASE = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
				var LOWER_CASE = "abcdefghijklmnopqrstuvwxyz";
				var NUMBER = "1234567890";
				var RAND_CODES = ALL_CASE + NUMBER;

				function getRand(type, length) {

					var key = [],
						times = type.length;
					for (let i = 0; i < length; i++) {
						key[i] = type[parseInt(Math.random() * times)];
					}

					return key.join(EMPTY_STRING);
				}

				return {
					getRandCode: function (length) {
						return getRand(RAND_CODES, length);
					},
					randAllCase: function (length) {
						return getRand(ALL_CASE, length);
					},
					randUpperCase: function (length) {
						return getRand(UPPER_CASE, length);
					},
					randLowerCase: function (length) {
						return getRand(LOWER_CASE, length);
					},
					randNumber: function (length) {
						length = length || 16;
						if (length > 16) {
							throw new Error("最多只能获取不超过 16 位的整数");
						} else {
							return parseInt(Math.random() * Math.pow(10, length));
						}
					},
					randNumberAndLetter: function (length) {
						var key = [],
							cases, num;
						switch (arguments[1]) {
							case 0:
								cases = ALL_CASE + NUMBER;
								num = 62;
								break;
							case 1:
								cases = UPPER_CASE + NUMBER;
								num = 36;
								break;
							case 2:
								cases = LOWER_CASE + NUMBER;
								num = 36;
								break;
							default:
								break;
						}
						for (let i = 0; i < length; i++) {
							key[i] = cases[parseInt(Math.random() * num)];
						}
						return key;
					},
					ALL_CASE: 0,
					UPPER_CASE: 1,
					LOWER_CASE: 2
				};
			})(),
			Collection: new Interface("Collection", // 接口名称
				["add", // add 方法，添加元素
					"clear", // clear 方法，清空元素
					"exists", // exists 方法，判断元素是否存在于集合中
					"existsAll", // existsAll 方法，判断元素集合是否都存在于集合中
					"equals", // equals 方法，判断元素是否和集合是否相同，由各个实现类各自实现具体需求
					"isEmpty", // isEmpty 方法，判断集合是否为空
					"remove", // remove 方法，删除某个指定的元素
					"size", // size 方法，获得集合的大小（元素个数）
					"toArray" // toArray 方法，将集合转化成数组
				]),
			Interface: {
				newInterface: function (name, methods) {
					if (arguments.length === 1) throw new Error("构建接口至少定义一个方法");

					if (!_isArray(methods)) {
						methods = array_slice.call(arguments, 1);
					}

					return new Interface(name, methods);
				},
				ensureImplements: function (object) {
					if (arguments.length < 2) throw new Error("判定对象是否是实现某接口，必须先提供至少一个被判定的接口");

					for (let i = 1, len = arguments.length; i < len; i++) {
						let target = arguments[i];
						if (!isInterface(target)) errorCast(object, Interface);

						let iterator = target.iterator();
						while (iterator.hasNext()) {
							let name = iterator.next();
							let method = object[name];
							if (!method || !typeIs(method, 'function')) noSuchMethod(name);
						}
					}
				},
				isInterface: isInterface
			},
			/**
			 * 迭代器
			 */
			iterator: function (obj) {
				return new Iterator(obj);
			}
		},
		dom: (function () {

			// 标签
			var START_START_TAG = '<',
				START_END_TAG = '</',
				END_TAG = '>',
				ENMPTY_END_TAG = ' />';

			// 包括 img 在内没有回标签的标签集合
			var NO_BODY_TAG = ['img', 'input', 'br', 'hr', 'title'];

			// 不能再在标签内部添加同名标签
			var NOT_SAME_TAG = [
				// HTML 体标签
				'html', 'head', 'title', 'body',
				// 其他标签
				'a', 'i', 'b', 'del', 'u', 'strike'
			];

			var ABEND_TAG = ['script'];

			const KEY_NAME = new Map();
			KEY_NAME.set('class', 'className');

			function getKeyName(key) {
				return KEY_NAME.has(key) ? KEY_NAME.get(key) : key;
			}

			function XmlWrapper(tag, attribute, xmlType) {

				let _hashcode = '$=!' + (2388 / Math.random() + Math.sin(Date.now()));

				if (!tag || !typeIs(tag, 'string')) throw new Error("只有非空文本能作为标签");
				if (ABEND_TAG.has(tag)) throw new Error("不允许使用 XmlWrapper 来动态构建 " + tag + " 元素");

				let attrs;
				if (attribute instanceof Map) {
					attrs = attribute;
				} else {
					attrs = new Map();
					if (attribute) {
						Object.forEach(attribute, function (key, value) {
							attrs.set(key, value);
						})
					}
				}
				var tIndex = 0,
					nIndex = 0, // 这两个数字不对外公开使用
					children = [],
					parent = null;

				this.add = function (element, parse) {

					if (!tag) {
						children.push({
							index: tIndex++,
							type: String,
							parse: (parse === undefined) ? true : parse
						})
					}

					if (NO_BODY_TAG.has(tag)) throw new Error("不能向当前标签（" + tag + "）内无法添加元素");

					if (element instanceof XmlWrapper) {
						if (NOT_SAME_TAG.has(tag) && tag === element.getTag()) throw new Error("当前 XML 节点的标签（" + tag + "）中不被允许储存同名子节点");

						children.push({
							index: nIndex++,
							type: XmlWrapper,
							element: element
						});
						element.setParent(this);
					} else if (typeIs(element, 'string')) {

						children.push({
							index: tIndex++,
							type: String,
							element: element,
							parse: (parse === undefined) ? true : parse
						});
					} else {
						unsupportedType(element);
					}

					return this;
				};

				this.clear = function () {

					this.clearChildren();
					attrs.clear();
					parent = null;
				};

				this.clearChildren = function () {
					nIndex = 0;
					tIndex = 0;
					children = [];
				};

				this.nodeCount = function (tagName) {
					if (!tagName) return nIndex;

					var count = 0;
					Object.forEach(children, function (i, element) {
						if (element.type === XmlWrapper &&
							element.element.getTag() === tagName) {
							count++;
						}
					});

					return count;
				};

				this.isEmpty = function () {
					return children.isEmpty();
				};

				/*
				 * xml = true html = false
				 */
				this.setXmlType = function (type) {
					xmlType = type;
				};
				this.setParent = function (p) {
					parent = p;
				};
				this.getParent = function () {
					return parent;
				};

				this.isRoot = function () {
					return parent == null;
				};

				this.getElement = function (index) {
					return children[index];
				};

				this.getText = function (index) {
					if (isNumber(index)) {
						if (index < tIndex) {
							for (let i = 0, len = children.length; i < len; i++) {
								let element = children[i];
								if (String === element.type && element.index === index) {
									return element.element;
								}
							}
						} else {
							indexOutOfBounds(tIndex, index);
						}
					} else {
						errorCast(index, Number);
					}
				};

				this.getTexts = function () {
					var results = [];
					for (let i = 0, len = children.length; i < len; i++) {
						let element = children[i];
						if (String === element.type) {
							results.push(element.element);
						}
					}
					return results;
				};

				this.hashCode = function () {
					return _hashcode;
				};

				this.getNode = function (index, tagName) {
					if (tagName) {
						if (index < nIndex) {
							for (let i = 0, len = children.length; i < len; i++) {
								let element = children[i];
								let node = element.element;
								if (element.type === XmlWrapper && index === element.index && node.getTag() === tagName) {
									return node;
								}
							}
						} else {
							indexOutOfBounds(nIndex, index);
						}
					} else if (isNumber(index)) {
						if (index < nIndex) {
							for (let i = 0, len = children.length; i < len; i++) {
								let element = children[i];
								if (element.type === XmlWrapper && index === element.index) {
									return element.element;
								}
							}
						} else {
							indexOutOfBounds(nIndex, index);
						}
					} else {
						errorCast(index, Number);
					}
					return null;
				};

				this.lastNode = function (tagName) {
					if (tagName) {
						if (typeIs(tagName, 'string')) {
							for (let i = children.length - 1; i >= 0; i--) {
								let element = children[i];
								let node = element.element;
								if (element.type === XmlWrapper && node.getTag() === tagName) {
									return node;
								}
							}
						} else {
							errorCast(tagName, String);
						}
					} else {
						for (let i = children.length - 1; i >= 0; i--) {
							let element = children[i];
							if (element.type === XmlWrapper) {
								return element.element;
							}
						}
					}
					return null;
				};
				this.getNodes = function (tagName) {

					let results = [];

					if (tagName) {
						if (typeIs(tagName, 'string')) {
							for (let i = 0, len = children.length; i < len; i++) {
								let element = children[i];
								let node = element.element;
								if (element.type === XmlWrapper && node.getTag() === tagName) {
									results.push(node);
								}
							}
						} else {
							errorCast(tagName, String);
						}
					} else {
						for (let i = 0, len = children.length; i < len; i++) {
							let element = children[i];
							if (element.type === XmlWrapper) {
								results.push(element.element);
							}
						}
					}
					return results;
				};

				this.size = function () {
					return children.length;
				};

				this.update = function (index, element) {

					if (!isNumber(index)) errorCast(index, Number);

					var type = typeof element;
					if (('string' !== type) && !(element instanceof XmlWrapper)) unsupportedType(element);

					var len = children.length;
					if (index >= len) indexOutOfBounds(index, len);

					var old = children[index];

					if (type === typeof old.element) {
						old.element = element;
						return;
					}
					old.element = element;

					var isFirst = true;
					for (index += 1; index < len; index++) {
						let tmp = children[index];
						if (type === typeof tmp.element) {
							if (isFirst) {
								old.index = tmp.index;
								isFirst = false;
							}
							tmp.index++;
						} else {
							tmp.index--;
						}
					}
					if ('string' === type) {
						nIndex++;
						tIndex--;
					} else {
						tIndex++;
						nIndex--;
					}
				};

				this.compare = function (another) {
					if (this === another) return true;
					if (!(another instanceof Map)) return false;
					if (tag !== another.getTag()) return false;

					for (let [key, val] of attrs.entries()) {
						var aVal = another.get(key);
						if (typeof val !== typeof aVal) return false;
						if (val !== aVal && // 字符串形式的文本节点或引用同一片内存空间
							!val.compare(aVal)) // 元素节点
							return false;
					}

					return true;
				}

				this.on = function (name, func) {
					browserOnly();
				}

				/*
				 * 将保存的对象渲染为一个dom结构
				 * 不同于 toString 转变为字符串，这个输出的 dom 结构是可以修改的，并且可以直接插入到 HTML 之中
				 */
				this.render = function () {

					browserOnly();

					var element = document.createElement(tag);

					attrs.forEach(function (val, key) {
						element[getKeyName(key)] = val;
					});

					Object.forEach(children, function (i, node) {
						switch (node.type) {
							case String:
								if (node.parse) {
									element.appendChild(document.createTextNode(node.element));
								} else {
									element.innerHTML = node.element;
								}
								break;
							case XmlWrapper:
								element.appendChild(node.element.render());
								break;
							default:
								unsupportedType(node.type);
						}
					});

					return element;
				}

				this.remove = function (index, count, type, tag) {

					var len = children.length;
					if (index < 0 || index >= len) indexOutOfBounds(index, 0);

					var last = index + count;
					if (len <= last) indexOutOfBounds(last, len);

					var removed = [];
					var nRemoved = 0,
						tRevmoed = 0;

					for (let i = index; i < last; i++) {

						let element = children[i];
						switch (element.type) {
							case String:
								if (type === 'string') {
									tRemoved++;
									removed.push(element.element);
								}
								break;
							case XmlWrapper:
								if (type === 'node') {
									if (!!tag && element.element.getTag() === tag) {
										removed.push(element.element);
										nRemoved++;
									} else if (!tag) {
										removed.push(element.element);
										nRemoved++;
									}
								}
								break;
							default:
								unsupportedType(element.type);
						}

					}
					tIndex -= tRevmoed;
					nIndex -= nRevmoed;

					for (let i = last; i < len; i++) {
						let element = children[i];
						switch (element.type) {
							case String:
								element.index -= tRevmoed;
								break;
							case XmlWrapper:
								element.index -= nRevmoed;
								break;
							default:
								unsupportedType(element.type);
						}
					}

					return removed;
				};
				this.putAttribute = function (key, val) {
					attrs.set(key, val);
				};
				this.putAllAttributes = function (obj) {
					Object.forEach(obj, function (key, val) {
						attrs.set(key, val);
					})
				};
				this.removeAttribute = function (key) {
					return attrs['delete'](key);
				};
				this.getAttribute = function (key) {
					return attrs.get(key);
				};
				this.containsAttribute = function (key) {
					return map.has(key);
				};

				this.getTag = function () {
					return tag;
				};
				this.toString = this.toHTML = function () {

					var xml = [START_START_TAG, tag];
					attrs.forEach(function (val, key) {
						xml.push(' ' + key + '="' + attrs.get(key) + '"');
					});

					var len = children.length;
					if (len === 0) {
						if (xmlType) {
							xml.push(ENMPTY_END_TAG);
						} else {
							if (NO_BODY_TAG.has(tag)) {
								xml.push(ENMPTY_END_TAG);
							} else {
								xml.push(END_TAG + START_END_TAG + tag + END_TAG);
							}
						}
					} else {
						xml.push(END_TAG);
						for (var i = 0; i < len; i++) {
							xml.push(children[i].element.toString());
						}
						xml.push(START_END_TAG + tag + END_TAG);
					}
					return xml.join(EMPTY_STRING);
				};
			}

			/**
			 * 通过制定格式的 JSON 对象创建一个 HTML 格式的 JSON：
			 * [{ tag : 'div', attribute : {
			 * 'class' : 'class_name' }, child : [ 'text', { tag : 'div', child : [
			 * 'div_text', { tag : 'input', attribute : { type : 'text' } } ] } ] }, {
			 * tag : 'p', attribute : { id : 'idname' }, child : [ { tag :
			 * 'span', child : [ 'spantext' ] } ] } ]
			 * 
			 * 说明： tag : 制定要创建的 HTML 节点的 tag attribute : HTML 节点的属性 child
			 * ：HTML节点的子元素（文本与子节点均通过 child 来表示）
			 */
			function jsonToHTML(json) {
				if (!_isArray(json)) {
					unsupportedType(json);
				}
				var result = [];
				for (let i = 0, len = json.length; i < len; i++) {
					let obj = json[i];
					if (!typeIs(obj, 'object')) {
						result.push((obj).toString());
					} else {
						let xml = new XmlWrapper(obj.tag, obj.attribute);
						let child = obj.child;
						if (child) {
							if (typeIs(child, 'string')) {
								xml.add(child);
							} else {
								var children = jsonToHTML(child);
								for (let j = 0, cLen = children.length; j < cLen; j++) {
									xml.add(children[j]);
								}
							}
						}
						result.push(xml);
					}
				}
				return result.join('');
			}

			// String -> XmlWrapper
			function parse(input) {

			}

			/**
			 * 将任意格式的 JSON 对象转换为一个 XML 字符串（不是 HTML）
			 */
			function jsonToXml(json, rootTag) {

				if (!typeIs(json, 'object', 'array', 'string', 'number')) unsupportedType(json);

				var root = new XmlWrapper(rootTag || 'root');
				root.setXmlType(true);
				if (typeOf(json, 'string')) {
					root.add(json);
				} else {
					for (let name in json) {
						if (hasOwnProperty(json, item)) {
							let item = json[name];
							root.add(jsonToXml(item, name));
						}
					}
				}

				return root;
			}

			return {
				newXmlWrapper: function (t, a, x) {
					return new XmlWrapper(t, a, x);
				},
				isXmlWrapper: function (obj) {
					return obj instanceof XmlWrapper;
				},
				jsonToHTML: jsonToHTML,
				jsonToXml: jsonToXml,
				/*
				 * 目前尚未实现该功能
				 */
				// parse : parse,
				// xmlToJSON : xmlToJSON
				XML: false,
				HTML: true
			};
		})(),
		encode: function () {

			var encodes = {},
				UTF8 = 'utf-8',
				GBK = 'gbk',
				UNICODE = 'unicode',
				BIG5 = 'big5';

			Object.defineProperty(encodes, 'UTF8', {
				value: UTF8,
				writable: false
			});
			Object.defineProperty(encodes, 'GBK', {
				value: GBK,
				writable: false
			});
			Object.defineProperty(encodes, 'UNICODE', {
				value: UNICODE,
				writable: false
			});
			Object.defineProperty(encodes, 'BIG5', {
				value: BIG5,
				writable: false
			});

			return encodes;
		},
		Validator: (function () {

			var UPPER_CASE = /[A-Z]/,
				LOWER_CASE = /[a-z]/,
				NUMBER = /[0-9]/,
				MARKS = /[\@#\$\%\&\*\!\^\+\=\-_\~:\;\,\.\?]/;

			function passwordCheckError(msg, id) {
				return {
					message: msg,
					id: id || 'password'
				};
			}

			function checkPassword(password) {

				if (password.isEmpty()) return passwordCheckError('密码不能为空');

				/*
				 * 密码校验必须符合以下两项规则
				 * 1. 长度必须 >= 6 位
				 */
				if (password.length < 6) return passwordCheckError('密码长度最少 6 位');

				/*
				 * 2. 必须包含大小写字母、数字、特殊字符（@#$%&*!^+=-_~:;,.?，不再左列中，以及不是字母、数字的字符被视为非法字符）中的三项
				 *  大写字母 小写字母 数字 特殊字符
				 *  0000 X 0
				 *  0001 X 1
				 *  0010 X 2
				 *  0011 X 3
				 *  0100 X 4
				 *  0101 X 5
				 *  0110 X 6
				 *  0111 O 7
				 *  1000 X 8
				 *  1001 X 9
				 *  1010 X 10
				 *  1011 O 11
				 *  1100 X 12
				 *  1101 O 13
				 *  1110 O 14
				 *  1111 O 15
				 */
				var point = 0;
				if (UPPER_CASE.test(password)) {
					point += 8;
				}
				if (LOWER_CASE.test(password)) {
					point += 4;
				}
				if (NUMBER.test(password)) {
					point += 2;
				}
				if (MARKS.test(password)) {
					point += 1;
				}
				if (point < 13 && point !== 7 && point !== 11)
					return passwordCheckError("密码至少要包含大写字母、小写字母、数字或特殊字符中的三项");

			}

			/*
			 * 身份证号验证
			 * 代码来源：http://www.cnblogs.com/xiaoafei1991/p/4309328.html
			 * 有修改
			 */
			function isIDNumber(cardid) {
				//身份证正则表达式(18位) 
				var isIdCard2 = /^[1-9]\d{5}(19\d{2}|[2-9]\d{3})((0\d)|(1[0-2]))(([0|1|2]\d)|3[0-1])(\d{4}|\d{3}X)$/i;
				var stard = "10X98765432"; //最后一位身份证的号码
				var first = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]; //1-17系数
				var sum = 0;
				if (!isIdCard2.test(cardid)) return false;

				var year = cardid.substr(6, 4),
					month = cardid.substr(10, 2),
					day = cardid.substr(12, 2);
				var birthday = cardid.substr(6, 8);
				if (birthday !== _calendar.formatTime(new Date(year, month, day), 'YYYYMMDD')) //校验日期是否合法
					return false;

				for (let i = 0; i < 17; i++) {
					sum += cardid[i] * first[i];
				}
				var result = sum % 11;
				var last = stard[result]; //计算出来的最后一位身份证号码

				return cardid.last().toUpperCase() === last.toUpperCase();
			}

			return {
				passwordCheckError: passwordCheckError,
				checkPassword: checkPassword,
				isIDNumber: isIDNumber
			};
		})(),
		constants : constants
	};

	that.Coralian = Coralian;
})();
